/*
 * AutomatonFromXMLParser.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "AutomatonFromXMLParser.h"

#include "../sax/ParserException.h"
#include "../alphabet/BlankSymbol.h"
#include "../label/Label.h"

#include "../XmlApi.hpp"

namespace automaton {

Automaton AutomatonFromXMLParser::parseAutomaton(std::list<sax::Token> &input) const {
	return parseAutomaton(input, std::set<FEATURES>({FEATURES::EPSILON_NFA, FEATURES::MULTI_INITIAL_STATE_NFA, FEATURES::NFA, FEATURES::DFA, FEATURES::COMPACT_NFA, FEATURES::EXTENDED_NFA, FEATURES::DPDA, FEATURES::SINGLE_POP_DPDA, FEATURES::INPUT_DRIVEN_NPDA, FEATURES::VISIBLY_PUSHDOWN_DPDA, FEATURES::VISIBLY_PUSHDOWN_NPDA, FEATURES::REAL_TIME_HEIGHT_DETERMINISTIC_DPDA,  FEATURES::REAL_TIME_HEIGHT_DETERMINISTIC_NPDA, FEATURES::NPDA, FEATURES::SINGLE_POP_NPDA, FEATURES::ONE_TAPE_DTM}));
}

Automaton AutomatonFromXMLParser::parseAutomaton(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(alib::xmlApi<EpsilonNFA>::first(input)) {
		if(!features.count(FEATURES::EPSILON_NFA)) throw exception::AlibException();
		return Automaton(parseEpsilonNFA(input));
	} else if(alib::xmlApi<MultiInitialStateNFA>::first(input)) {
		if(!features.count(FEATURES::MULTI_INITIAL_STATE_NFA)) throw exception::AlibException();
		return Automaton(parseMultiInitialStateNFA(input));
	} else if(alib::xmlApi<NFA>::first(input)) {
		if(!features.count(FEATURES::NFA)) throw exception::AlibException();
		return Automaton(parseNFA(input));
	} else if(alib::xmlApi<DFA>::first(input)) {
		if(!features.count(FEATURES::DFA)) throw exception::AlibException();
		return Automaton(parseDFA(input));
	} else if(alib::xmlApi<CompactNFA>::first(input)) {
		if(!features.count(FEATURES::COMPACT_NFA)) throw exception::AlibException();
		return Automaton(parseCompactNFA(input));
	} else if(alib::xmlApi<ExtendedNFA>::first(input)) {
		if(!features.count(FEATURES::EXTENDED_NFA)) throw exception::AlibException();
		return Automaton(parseExtendedNFA(input));
	} else if(alib::xmlApi<DPDA>::first(input)) {
		if(!features.count(FEATURES::DPDA)) throw exception::AlibException();
		return Automaton(parseDPDA(input));
	} else if(alib::xmlApi<SinglePopDPDA>::first(input)) {
		if(!features.count(FEATURES::SINGLE_POP_DPDA)) throw exception::AlibException();
		return Automaton(parseSinglePopDPDA(input));
	} else if(alib::xmlApi<InputDrivenNPDA>::first(input)) {
		if(!features.count(FEATURES::INPUT_DRIVEN_NPDA)) throw exception::AlibException();
		return Automaton(parseInputDrivenNPDA(input));
	} else if(alib::xmlApi<VisiblyPushdownDPDA>::first(input)) {
		if(!features.count(FEATURES::VISIBLY_PUSHDOWN_DPDA)) throw exception::AlibException();
		return Automaton(parseVisiblyPushdownDPDA(input));
	} else if(alib::xmlApi<VisiblyPushdownNPDA>::first(input)) {
		if(!features.count(FEATURES::VISIBLY_PUSHDOWN_NPDA)) throw exception::AlibException();
		return Automaton(parseVisiblyPushdownNPDA(input));
	} else if(alib::xmlApi<RealTimeHeightDeterministicDPDA>::first(input)) {
		if(!features.count(FEATURES::REAL_TIME_HEIGHT_DETERMINISTIC_DPDA)) throw exception::AlibException();
		return Automaton(parseRealTimeHeightDeterministicDPDA(input));
	} else if(alib::xmlApi<RealTimeHeightDeterministicNPDA>::first(input)) {
		if(!features.count(FEATURES::REAL_TIME_HEIGHT_DETERMINISTIC_NPDA)) throw exception::AlibException();
		return Automaton(parseRealTimeHeightDeterministicNPDA(input));
	} else if(alib::xmlApi<NPDA>::first(input)) {
		if(!features.count(FEATURES::NPDA)) throw exception::AlibException();
		return Automaton(parseNPDA(input));
	} else if(alib::xmlApi<SinglePopNPDA>::first(input)) {
		if(!features.count(FEATURES::SINGLE_POP_NPDA)) throw exception::AlibException();
		return Automaton(parseSinglePopNPDA(input));
	} else if(alib::xmlApi<OneTapeDTM>::first(input)) {
		if(!features.count(FEATURES::ONE_TAPE_DTM)) throw exception::AlibException();
		return Automaton(parseOneTapeDTM(input));
	} else
		throw sax::ParserException(sax::Token("Automaton / EpsilonNFA / NFA / DFA / CompactNFA / ExtendedNFA / DPDA / SinglePopDPDA / InputDrivenNPDA / VisiblyPushdownDPDA / VisiblyPushdownNPDA / RealTimeHeightDeterministicDPDA / RealTimeHeightDeterministicNPDA / NPDA / SinglePopNPDA / OneTapeDTM", sax::Token::TokenType::START_ELEMENT), input.front());
}

bool AutomatonFromXMLParser::first(const std::list<sax::Token>& input) const {
	if(alib::xmlApi<EpsilonNFA>::first(input) || alib::xmlApi<NFA>::first(input) || alib::xmlApi<DFA>::first(input) || alib::xmlApi<CompactNFA>::first(input) || alib::xmlApi<ExtendedNFA>::first(input) || alib::xmlApi<DPDA>::first(input) || alib::xmlApi<SinglePopDPDA>::first(input)  || alib::xmlApi<InputDrivenNPDA>::first(input) || alib::xmlApi<VisiblyPushdownDPDA>::first(input)  || alib::xmlApi<VisiblyPushdownNPDA>::first(input) || alib::xmlApi<RealTimeHeightDeterministicDPDA>::first(input) || alib::xmlApi<RealTimeHeightDeterministicNPDA>::first(input) || alib::xmlApi<NPDA>::first(input) || alib::xmlApi<SinglePopNPDA>::first(input) || alib::xmlApi<OneTapeDTM>::first(input)) {
		return true;
	} else {
		return false;
	}
}

template<class T>
void AutomatonFromXMLParser::parseTransitions(std::list<sax::Token> &input, T& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transitions");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		parseTransition(input, automaton);
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transitions");
}

DFA AutomatonFromXMLParser::parseDFA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_DFA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	State initialState = parseInitialState(input);
	std::set<State> finalStates = parseFinalStates(input);

	DFA automaton(initialState);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<DFA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_DFA);
	return automaton;
}

NFA AutomatonFromXMLParser::parseNFA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_NFA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	State initialState = parseInitialState(input);
	std::set<State> finalStates = parseFinalStates(input);

	NFA automaton(initialState);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<NFA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_NFA);
	return automaton;
}

MultiInitialStateNFA AutomatonFromXMLParser::parseMultiInitialStateNFA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_MULTI_INITIAL_STATE_NFA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<State> initialStates = parseInitialStates(input);
	std::set<State> finalStates = parseFinalStates(input);

	MultiInitialStateNFA automaton;
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setInitialStates(initialStates);
	automaton.setFinalStates(finalStates);

	parseTransitions<MultiInitialStateNFA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_MULTI_INITIAL_STATE_NFA);
	return automaton;
}

EpsilonNFA AutomatonFromXMLParser::parseEpsilonNFA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_EPSILON_NFA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	State initialState = parseInitialState(input);
	std::set<State> finalStates = parseFinalStates(input);

	EpsilonNFA automaton(initialState);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<EpsilonNFA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_EPSILON_NFA);
	return automaton;
}

CompactNFA AutomatonFromXMLParser::parseCompactNFA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_COMPACT_NFA);
	
	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	State initialState = parseInitialState(input);
	std::set<State> finalStates = parseFinalStates(input);

	CompactNFA automaton(initialState);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<CompactNFA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_COMPACT_NFA);
	return automaton;
}

ExtendedNFA AutomatonFromXMLParser::parseExtendedNFA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_EXTENDED_NFA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	State initialState = parseInitialState(input);
	std::set<State> finalStates = parseFinalStates(input);

	ExtendedNFA automaton(initialState);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<ExtendedNFA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_EXTENDED_NFA);
	return automaton;
}

DPDA AutomatonFromXMLParser::parseDPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_DPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	State initialState = parseInitialState(input);
	alphabet::Symbol initialStackSymbol = parseInitialStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	DPDA automaton(initialState, initialStackSymbol);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<DPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_DPDA);

	return automaton;
}

SinglePopDPDA AutomatonFromXMLParser::parseSinglePopDPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_SINGLE_POP_DPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	State initialState = parseInitialState(input);
	alphabet::Symbol initialStackSymbol = parseInitialStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	SinglePopDPDA automaton(initialState, initialStackSymbol);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<SinglePopDPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_SINGLE_POP_DPDA);

	return automaton;
}

InputDrivenNPDA AutomatonFromXMLParser::parseInputDrivenNPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_INPUT_DRIVEN_NPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	State initialState = parseInitialState(input);
	alphabet::Symbol initialStackSymbol = parseInitialStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	InputDrivenNPDA automaton(initialState, initialStackSymbol);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setFinalStates(finalStates);

	parseInputToPushdownStoreOperation(input, automaton);
	parseTransitions<InputDrivenNPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_INPUT_DRIVEN_NPDA);

	return automaton;
}

VisiblyPushdownDPDA AutomatonFromXMLParser::parseVisiblyPushdownDPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_VISIBLY_PUSHDOWN_DPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> callInputAlphabet = parseCallInputAlphabet(input);
	std::set<alphabet::Symbol> returnInputAlphabet = parseReturnInputAlphabet(input);
	std::set<alphabet::Symbol> localInputAlphabet = parseLocalInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	State initialState = parseInitialState(input);
	alphabet::Symbol bottomOfTheStackSymbol = parseBottomOfTheStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	VisiblyPushdownDPDA automaton(initialState, bottomOfTheStackSymbol);
	automaton.setStates(states);
	automaton.setCallInputAlphabet(callInputAlphabet);
	automaton.setReturnInputAlphabet(returnInputAlphabet);
	automaton.setLocalInputAlphabet(localInputAlphabet);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<VisiblyPushdownDPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_VISIBLY_PUSHDOWN_DPDA);

	return automaton;
}

VisiblyPushdownNPDA AutomatonFromXMLParser::parseVisiblyPushdownNPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_VISIBLY_PUSHDOWN_NPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> callInputAlphabet = parseCallInputAlphabet(input);
	std::set<alphabet::Symbol> returnInputAlphabet = parseReturnInputAlphabet(input);
	std::set<alphabet::Symbol> localInputAlphabet = parseLocalInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	std::set<State> initialStates = parseInitialStates(input);
	alphabet::Symbol bottomOfTheStackSymbol = parseBottomOfTheStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	VisiblyPushdownNPDA automaton(bottomOfTheStackSymbol);
	automaton.setStates(states);
	automaton.setCallInputAlphabet(callInputAlphabet);
	automaton.setReturnInputAlphabet(returnInputAlphabet);
	automaton.setLocalInputAlphabet(localInputAlphabet);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setInitialStates(initialStates);
	automaton.setFinalStates(finalStates);

	parseTransitions<VisiblyPushdownNPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_VISIBLY_PUSHDOWN_NPDA);

	return automaton;
}

RealTimeHeightDeterministicDPDA AutomatonFromXMLParser::parseRealTimeHeightDeterministicDPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_DPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	State initialState = parseInitialState(input);
	alphabet::Symbol bottomOfTheStackSymbol = parseBottomOfTheStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	RealTimeHeightDeterministicDPDA automaton(initialState, bottomOfTheStackSymbol);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<RealTimeHeightDeterministicDPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_DPDA);

	return automaton;
}

RealTimeHeightDeterministicNPDA AutomatonFromXMLParser::parseRealTimeHeightDeterministicNPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_NPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	std::set<State> initialStates = parseInitialStates(input);
	alphabet::Symbol bottomOfTheStackSymbol = parseBottomOfTheStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	RealTimeHeightDeterministicNPDA automaton(bottomOfTheStackSymbol);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setInitialStates(initialStates);
	automaton.setFinalStates(finalStates);

	parseTransitions<RealTimeHeightDeterministicNPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_NPDA);

	return automaton;
}

NPDA AutomatonFromXMLParser::parseNPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_NPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	State initialState = parseInitialState(input);
	alphabet::Symbol initialStackSymbol = parseInitialStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	NPDA automaton(initialState, initialStackSymbol);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<NPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_NPDA);

	return automaton;
}

SinglePopNPDA AutomatonFromXMLParser::parseSinglePopNPDA(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_SINGLE_POP_NPDA);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<alphabet::Symbol> stackSymbols = parseStackAlphabet(input);
	State initialState = parseInitialState(input);
	alphabet::Symbol initialStackSymbol = parseInitialStackSymbol(input);
	std::set<State> finalStates = parseFinalStates(input);

	SinglePopNPDA automaton(initialState, initialStackSymbol);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setStackAlphabet(stackSymbols);
	automaton.setFinalStates(finalStates);

	parseTransitions<SinglePopNPDA>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_SINGLE_POP_NPDA);

	return automaton;
}

OneTapeDTM AutomatonFromXMLParser::parseOneTapeDTM(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::AUTOMATON_ONE_TAPE_DTM);

	std::set<State> states = parseStates(input);
	std::set<alphabet::Symbol> inputSymbols = parseInputAlphabet(input);
	std::set<alphabet::Symbol> tapeSymbols = parseTapeAlphabet(input);
	alphabet::Symbol blank = parseBlankSymbol(input);
	State initialState = parseInitialState(input);
	std::set<State> finalStates = parseFinalStates(input);

	OneTapeDTM automaton(initialState, blank);
	automaton.setStates(states);
	automaton.setInputAlphabet(inputSymbols);
	automaton.setTapeAlphabet(tapeSymbols);
	automaton.setInitialState(initialState);
	automaton.setFinalStates(finalStates);

	parseTransitions<OneTapeDTM>(input, automaton);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::AUTOMATON_ONE_TAPE_DTM);
	return automaton;
}

std::set<State> AutomatonFromXMLParser::parseStates(std::list<sax::Token> &input) const {
	std::set<State> states;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "states");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		states.insert(parseState(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "states");
	return states;
}

State AutomatonFromXMLParser::parseState(std::list<sax::Token> &input) const {
	return State(alib::xmlApi<label::Label>::parse(input));
}

std::set<alphabet::Symbol> AutomatonFromXMLParser::parseCallInputAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> inputSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "callInputAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		inputSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "callInputAlphabet");
	return inputSymbols;
}

std::set<alphabet::Symbol> AutomatonFromXMLParser::parseReturnInputAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> inputSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "returnInputAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		inputSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "returnInputAlphabet");
	return inputSymbols;
}

std::set<alphabet::Symbol> AutomatonFromXMLParser::parseLocalInputAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> inputSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "localInputAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		inputSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "localInputAlphabet");
	return inputSymbols;
}

std::set<alphabet::Symbol> AutomatonFromXMLParser::parseInputAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> inputSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "inputAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		inputSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "inputAlphabet");
	return inputSymbols;
}

State AutomatonFromXMLParser::parseInitialState(std::list<sax::Token> &input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "initialState");
	State state(alib::xmlApi<label::Label>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "initialState");
	return state;
}

std::set<State> AutomatonFromXMLParser::parseInitialStates(std::list<sax::Token> &input) const {
	std::set<State> initialStates;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "initialStates");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		initialStates.insert(State(alib::xmlApi<label::Label>::parse(input)));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "initialStates");
	return initialStates;
}

std::set<State> AutomatonFromXMLParser::parseFinalStates(std::list<sax::Token> &input) const {
	std::set<State> finalStates;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "finalStates");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		finalStates.insert(State(alib::xmlApi<label::Label>::parse(input)));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "finalStates");
	return finalStates;
}

std::set<alphabet::Symbol> AutomatonFromXMLParser::parseStackAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> stackSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "stackAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		stackSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "stackAlphabet");
	return stackSymbols;
}

std::set<alphabet::Symbol> AutomatonFromXMLParser::parseInitialStackSymbols(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> initialSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "initialStackSymbols");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		initialSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "initialStackSymbols");
	return initialSymbols;
}

alphabet::Symbol AutomatonFromXMLParser::parseInitialStackSymbol(std::list<sax::Token> &input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "initialStackSymbol");
	alphabet::Symbol initialSymbol(alib::xmlApi<alphabet::Symbol>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "initialStackSymbol");
	return initialSymbol;
}

std::set<alphabet::Symbol> AutomatonFromXMLParser::parseTapeAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> tapeSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "tapeAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		tapeSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "tapeAlphabet");
	return tapeSymbols;
}

alphabet::Symbol AutomatonFromXMLParser::parseBlankSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "blankSymbol");
	alphabet::Symbol blank(alib::xmlApi<alphabet::Symbol>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "blankSymbol");
	return blank;
}

alphabet::Symbol AutomatonFromXMLParser::parseBottomOfTheStackSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "bottomOfTheStackSymbol");
	alphabet::Symbol blank(alib::xmlApi<alphabet::Symbol>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "bottomOfTheStackSymbol");
	return blank;
}

void AutomatonFromXMLParser::parseInputToPushdownStoreOperation(std::list<sax::Token>& input, automaton::InputDrivenNPDA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "inputToPushdownStoreOperations");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "operation");

		alphabet::Symbol inputSymbol(alib::xmlApi<alphabet::Symbol>::parse(input));

		std::vector<alphabet::Symbol> pop = parseTransitionPop(input);
		std::vector<alphabet::Symbol> push = parseTransitionPush(input);

		automaton.setPushdownStoreOperation(inputSymbol, pop, push);

		popToken(input, sax::Token::TokenType::END_ELEMENT, "operation");
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "inputToPushdownStoreOperations");
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, DFA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
	State to = parseTransitionTo(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, to);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, MultiInitialStateNFA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
	State to = parseTransitionTo(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, to);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, NFA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
	State to = parseTransitionTo(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, to);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, EpsilonNFA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant = parseTransitionInputEpsilonSymbol(input);
	State to = parseTransitionTo(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputVariant, to);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, CompactNFA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	string::LinearString inputString = parseTransitionInputString(input);
	State to = parseTransitionTo(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputString, to);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, ExtendedNFA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	regexp::RegExp inputRegexp = parseTransitionInputRegexp(input);
	State to = parseTransitionTo(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputRegexp, to);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, DPDA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
	std::vector<alphabet::Symbol> pop = parseTransitionPop(input);
	State to = parseTransitionTo(input);
	std::vector<alphabet::Symbol> push = parseTransitionPush(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, pop, to, push);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, SinglePopDPDA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
	alphabet::Symbol pop = parseTransitionSinglePop(input);
	State to = parseTransitionTo(input);
	std::vector<alphabet::Symbol> push = parseTransitionPush(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, pop, to, push);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, InputDrivenNPDA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
	State to = parseTransitionTo(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, to);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, VisiblyPushdownDPDA& automaton) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "callTransition")) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "callTransition");
		State from = parseTransitionFrom(input);
		alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
		State to = parseTransitionTo(input);
		alphabet::Symbol push = parseTransitionSinglePush(input);

		automaton.addCallTransition(from, inputSymbol, to, push);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "callTransition");
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "returnTransition")) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "returnTransition");
		State from = parseTransitionFrom(input);
		alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
		alphabet::Symbol pop = parseTransitionSinglePop(input);
		State to = parseTransitionTo(input);

		automaton.addReturnTransition(from, inputSymbol, pop, to);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "returnTransition");
	} else {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "localTransition");
		State from = parseTransitionFrom(input);
		alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
		State to = parseTransitionTo(input);

		automaton.addLocalTransition(from, inputSymbol, to);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "localTransition");
	}
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, VisiblyPushdownNPDA& automaton) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "callTransition")) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "callTransition");
		State from = parseTransitionFrom(input);
		alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
		State to = parseTransitionTo(input);
		alphabet::Symbol push = parseTransitionSinglePush(input);

		automaton.addCallTransition(from, inputSymbol, to, push);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "callTransition");
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "returnTransition")) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "returnTransition");
		State from = parseTransitionFrom(input);
		alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
		alphabet::Symbol pop = parseTransitionSinglePop(input);
		State to = parseTransitionTo(input);

		automaton.addReturnTransition(from, inputSymbol, pop, to);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "returnTransition");
	} else {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "localTransition");
		State from = parseTransitionFrom(input);
		alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
		State to = parseTransitionTo(input);

		automaton.addLocalTransition(from, inputSymbol, to);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "localTransition");
	}
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, RealTimeHeightDeterministicDPDA& automaton) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "callTransition")) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "callTransition");
		State from = parseTransitionFrom(input);
		std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
		State to = parseTransitionTo(input);
		alphabet::Symbol push = parseTransitionSinglePush(input);

		automaton.addCallTransition(from, inputSymbol, to, push);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "callTransition");
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "returnTransition")) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "returnTransition");
		State from = parseTransitionFrom(input);
		std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
		alphabet::Symbol pop = parseTransitionSinglePop(input);
		State to = parseTransitionTo(input);

		automaton.addReturnTransition(from, inputSymbol, pop, to);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "returnTransition");
	} else {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "localTransition");
		State from = parseTransitionFrom(input);
		std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
		State to = parseTransitionTo(input);

		automaton.addLocalTransition(from, inputSymbol, to);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "localTransition");
	}
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, RealTimeHeightDeterministicNPDA& automaton) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "callTransition")) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "callTransition");
		State from = parseTransitionFrom(input);
		std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
		State to = parseTransitionTo(input);
		alphabet::Symbol push = parseTransitionSinglePush(input);

		automaton.addCallTransition(from, inputSymbol, to, push);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "callTransition");
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "returnTransition")) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "returnTransition");
		State from = parseTransitionFrom(input);
		std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
		alphabet::Symbol pop = parseTransitionSinglePop(input);
		State to = parseTransitionTo(input);

		automaton.addReturnTransition(from, inputSymbol, pop, to);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "returnTransition");
	} else {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "localTransition");
		State from = parseTransitionFrom(input);
		std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
		State to = parseTransitionTo(input);

		automaton.addLocalTransition(from, inputSymbol, to);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "localTransition");
	}
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, NPDA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
	std::vector<alphabet::Symbol> pop = parseTransitionPop(input);
	State to = parseTransitionTo(input);
	std::vector<alphabet::Symbol> push = parseTransitionPush(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, pop, to, push);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, SinglePopNPDA& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	std::variant<string::Epsilon, alphabet::Symbol> inputSymbol = parseTransitionInputEpsilonSymbol(input);
	alphabet::Symbol pop = parseTransitionSinglePop(input);
	State to = parseTransitionTo(input);
	std::vector<alphabet::Symbol> push = parseTransitionPush(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, pop, to, push);
}

void AutomatonFromXMLParser::parseTransition(std::list<sax::Token>& input, OneTapeDTM& automaton) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "transition");
	State from = parseTransitionFrom(input);
	alphabet::Symbol inputSymbol = parseTransitionInputSymbol(input);
	State to = parseTransitionTo(input);
	alphabet::Symbol outputSymbol = parseTransitionOutputSymbol(input);
	Shift shift = parseTransitionShift(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "transition");

	automaton.addTransition(from, inputSymbol, to, outputSymbol, shift);
}

State AutomatonFromXMLParser::parseTransitionTo(std::list<sax::Token> &input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "to");
	State state(alib::xmlApi<label::Label>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "to");
	return state;
}

State AutomatonFromXMLParser::parseTransitionFrom(std::list<sax::Token> &input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "from");
	State state(alib::xmlApi<label::Label>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "from");
	return state;
}

Shift AutomatonFromXMLParser::parseTransitionShift(std::list<sax::Token>& input) const {
	Shift shift;

	popToken(input, sax::Token::TokenType::START_ELEMENT, "shift");
	if (isToken(input, sax::Token::TokenType::CHARACTER, "left")) {
		shift = LEFT;
	} else if (isToken(input, sax::Token::TokenType::CHARACTER, "right")) {
		shift = RIGHT;
	} else if (isToken(input, sax::Token::TokenType::CHARACTER, "none")) {
		shift = NONE;
	} else {
		throw sax::ParserException(sax::Token("", sax::Token::TokenType::CHARACTER), input.front());
	}
	input.pop_front();
	popToken(input, sax::Token::TokenType::END_ELEMENT, "shift");

	return shift;
}

std::vector<alphabet::Symbol> AutomatonFromXMLParser::parseTransitionPop(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> pops;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "pop");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		pops.push_back(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "pop");
	return pops;
}

alphabet::Symbol AutomatonFromXMLParser::parseTransitionSinglePop(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "pop");
	alphabet::Symbol pop = alib::xmlApi<alphabet::Symbol>::parse(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "pop");
	return pop;
}

std::vector<alphabet::Symbol> AutomatonFromXMLParser::parseTransitionPush(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> pushes;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "push");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		pushes.push_back(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "push");
	return pushes;
}

alphabet::Symbol AutomatonFromXMLParser::parseTransitionSinglePush(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "push");
	alphabet::Symbol push = alib::xmlApi<alphabet::Symbol>::parse(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "push");
	return push;
}

alphabet::Symbol AutomatonFromXMLParser::parseTransitionInputSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "input");
	alphabet::Symbol result(alib::xmlApi<alphabet::Symbol>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "input");
	return result;
}

alphabet::Symbol AutomatonFromXMLParser::parseTransitionOutputSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "output");
	alphabet::Symbol result(alib::xmlApi<alphabet::Symbol>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "output");
	return result;
}

std::variant<string::Epsilon, alphabet::Symbol> AutomatonFromXMLParser::parseTransitionInputEpsilonSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "input");
	
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "epsilon")) {
		std::variant<string::Epsilon, alphabet::Symbol> result(string::Epsilon::EPSILON);
		input.pop_front();
		popToken(input, sax::Token::TokenType::END_ELEMENT, "epsilon");
		popToken(input, sax::Token::TokenType::END_ELEMENT, "input");
		return result;
	} else {
		std::variant<string::Epsilon, alphabet::Symbol> result(alib::xmlApi<alphabet::Symbol>::parse(input));
		popToken(input, sax::Token::TokenType::END_ELEMENT, "input");
		return result;
	}
}

std::variant<string::Epsilon, alphabet::Symbol> AutomatonFromXMLParser::parseTransitionOutputEpsilonSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "output");
	
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "epsilon")) {
		std::variant<string::Epsilon, alphabet::Symbol> result(string::Epsilon::EPSILON);
		input.pop_front();
		popToken(input, sax::Token::TokenType::END_ELEMENT, "epsilon");
		popToken(input, sax::Token::TokenType::END_ELEMENT, "output");
		return result;
	} else {
		std::variant<string::Epsilon, alphabet::Symbol> result(alib::xmlApi<alphabet::Symbol>::parse(input));
		popToken(input, sax::Token::TokenType::END_ELEMENT, "output");
		return result;
	}
}

string::LinearString AutomatonFromXMLParser::parseTransitionInputString(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "input");
	string::LinearString result(alib::xmlApi<string::LinearString>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "input");
	return result;
}

regexp::RegExp AutomatonFromXMLParser::parseTransitionInputRegexp(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "input");
	regexp::RegExp result(alib::xmlApi<regexp::RegExp>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "input");
	return result;
}

} /* namespace automaton */

