/*
 * DFA.h
 *
 *  Created on: Mar 25, 2013
 *      Author: Jan Travnicek
 */

#ifndef DFA_H_
#define DFA_H_

#include <map>
#include "../AutomatonBase.h"
#include "../common/SingleInitialState.h"
#include "../common/InputAlphabet.h"
#include "../../alphabet/Symbol.h"

namespace automaton {

/**
 * Represents Finite Automaton.
 * Can store nondeterministic finite automaton without epsilon transitions.
 */
class DFA : public std::acceptor<DFA, VisitableAutomatonBase, std::acceptor<DFA, alib::VisitableObjectBase, AutomatonBase> >, public SingleInitialState, public InputAlphabet {
protected:
	std::map<std::pair<State, alphabet::Symbol>, State> transitions;
public:
	explicit DFA(const State& initialState);

	virtual AutomatonBase* clone() const;
	
	virtual AutomatonBase* plunder() &&;

	/**
	 * @copydoc Automaton::removeState(const State&)
	 */
	virtual bool removeState(const State& state);

	/**
	 * @copydoc Automaton::removeInputSymbol(const Symbol&)
	 */
	virtual bool removeInputSymbol(const alphabet::Symbol& symbol);
	
	/**
	 * Adds transition defined by parameters to the automaton.
	 * @param current current state
	 * @param input input symbol
	 * @param next next state
	 * @throws AutomatonException when transition already exists or when transition contains state or symbol not present in the automaton
	 */
	bool addTransition(const State& current, const alphabet::Symbol& input, const State& next);

	/**
	 * Removes transition from the automaton.
	 * @param transition transition to remove
	 * @throws AutomatonException when transition doesn't exists.
	 */
	bool removeTransition(const State& current, const alphabet::Symbol& input, const State& next);

	/**
	 * @return automaton transitions
	 */
	const std::map<std::pair<State, alphabet::Symbol>, State>& getTransitions() const;

	/**
	 * @return automaton transitions from state
	 */
	std::map<std::pair<State, alphabet::Symbol>, State> getTransitionsFromState(const State& from) const;
	
	/**
	 * @return automaton transitions to state
	 */
	std::map<std::pair<State, alphabet::Symbol>, State> getTransitionsToState(const State& from) const;

	/**
	 * Determines whether DFA is total deterministic
	 * FA is deterministic if and only if
	 * \li \c it is deterministic. (Trivial here)
	 * \li \c size of transition function \forall states and \forall input symbols \delta (from state, input symbol) = 1
	 * @return true when automaton is total deterministic, false otherwise
	 */
	bool isTotal() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const DFA& other) const;
	
	virtual void operator>>(std::ostream& os) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<DFA>();
	}
};

} /* namespace automaton */

namespace std {

template<>
struct compare<automaton::DFA> {
	int operator()(const automaton::DFA& first, const automaton::DFA& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* DFA_H_ */
