/*
 * RealTimeHeightDeterministicDPDA.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#include "RealTimeHeightDeterministicDPDA.h"
#include "../../std/map.hpp"
#include "../AutomatonException.h"
#include <algorithm>
#include <sstream>

namespace automaton {

RealTimeHeightDeterministicDPDA::RealTimeHeightDeterministicDPDA(const State& initialState, const alphabet::Symbol& bottomOfTheStackSymbol) : SingleInitialState(initialState), BottomOfTheStackSymbolPushdownStoreAlphabet(bottomOfTheStackSymbol) {

}

AutomatonBase* RealTimeHeightDeterministicDPDA::clone() const {
	return new RealTimeHeightDeterministicDPDA(*this);
}

AutomatonBase* RealTimeHeightDeterministicDPDA::plunder() && {
	return new RealTimeHeightDeterministicDPDA(std::move(*this));
}

bool RealTimeHeightDeterministicDPDA::removeState(const State& state) {
	if (initialState == state) {
		throw AutomatonException("State \"" + (std::string) state.getName() + "\" is initial state.");
	}

	if (finalStates.find(state) != finalStates.end()) {
		throw AutomatonException("State \"" + (std::string) state.getName() + "\" is final state.");
	}

	for (const std::pair<std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>>, std::pair<State, alphabet::Symbol> >& callTransition : callTransitions) {
		if (state == callTransition.first.first)
			throw AutomatonException("State \"" + (std::string) state.getName() + "\" is used in transition.");
		if(callTransition.second.first == state)
			throw AutomatonException("State \"" + (std::string) state.getName() + "\" is used in transition.");
	}

	for (const std::pair<std::tuple<State, std::variant<string::Epsilon, alphabet::Symbol>, alphabet::Symbol>, State>& returnTransition : returnTransitions) {
		if (state == std::get<0>(returnTransition.first))
			throw AutomatonException("State \"" + (std::string) state.getName() + "\" is used in transition.");
		if(returnTransition.second == state)
			throw AutomatonException("State \"" + (std::string) state.getName() + "\" is used in transition.");
	}

	for (const std::pair<std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>>, State>& localTransition : localTransitions) {
		if (state == localTransition.first.first)
			throw AutomatonException("State \"" + (std::string) state.getName() + "\" is used in transition.");
		if(localTransition.second == state)
			throw AutomatonException("State \"" + (std::string) state.getName() + "\" is used in transition.");
	}

	return states.erase(state);
}

bool RealTimeHeightDeterministicDPDA::removeInputSymbol(const alphabet::Symbol& symbol) {
	for (const std::pair<std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>>, std::pair<State, alphabet::Symbol> >& callTransition : callTransitions) {
		if (callTransition.first.second.is<alphabet::Symbol>() && symbol == callTransition.first.second.get<alphabet::Symbol>())
			throw AutomatonException("Input symbol \"" + (std::string) symbol + "\" is used in transition.");
	}

	for (const std::pair<std::tuple<State, std::variant<string::Epsilon, alphabet::Symbol>, alphabet::Symbol>, State>& returnTransition : returnTransitions) {
		if (std::get<1>(returnTransition.first).is<alphabet::Symbol>() && symbol == std::get<1>(returnTransition.first).get<alphabet::Symbol>())
			throw AutomatonException("Input symbol \"" + (std::string) symbol + "\" is used in transition.");
	}

	for (const std::pair<std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>>, State>& localTransition : localTransitions) {
		if (localTransition.first.second.is<alphabet::Symbol>() && symbol == localTransition.first.second.get<alphabet::Symbol>())
			throw AutomatonException("Input symbol \"" + (std::string) symbol + "\" is used in transition.");
	}

	return inputAlphabet.erase(symbol);
}

bool RealTimeHeightDeterministicDPDA::removeStackSymbol(const alphabet::Symbol& symbol) {
	for (const std::pair<std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>>, std::pair<State, alphabet::Symbol> >& callTransition : callTransitions) {
		if (symbol == callTransition.second.second)
			throw AutomatonException("Stack symbol \"" + (std::string) symbol + "\" is used in transition.");
	}

	for (const std::pair<std::tuple<State, std::variant<string::Epsilon, alphabet::Symbol>, alphabet::Symbol>, State>& returnTransition : returnTransitions) {
		if (symbol == std::get<2>(returnTransition.first))
			throw AutomatonException("Stack symbol \"" + (std::string) symbol + "\" is used in transition.");
	}

	if(bottomOfTheStackSymbol == symbol) {
		throw AutomatonException("Stack symbol \"" + (std::string) symbol + "\" is start symbol.");
	}

	return stackAlphabet.erase(symbol);
}

bool RealTimeHeightDeterministicDPDA::addCallTransition(const State& from, const std::variant<string::Epsilon, alphabet::Symbol>& input, const State& to, const alphabet::Symbol& push) {
	if (states.find(from) == states.end()) {
		throw AutomatonException("State \"" + (std::string) from.getName() + "\" doesn't exist.");
	}

	if (input.is<alphabet::Symbol>() && inputAlphabet.find(input.get<alphabet::Symbol>()) == inputAlphabet.end()) {
		throw AutomatonException("Input symbol \"" + (std::string) input.get<alphabet::Symbol>() + "\" doesn't exist.");
	}

	if (states.find(to) == states.end()) {
		throw AutomatonException("State \"" + (std::string) to.getName() + "\" doesn't exist.");
	}

	if (stackAlphabet.find(push) == stackAlphabet.end()) {
		throw AutomatonException("Stack symbol \"" + (std::string) push + "\" doesn't exist.");
	}

	std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>> key(from, input);
	std::pair<automaton::State, alphabet::Symbol> value = std::make_pair(to, push);

	if(callTransitions.find(key) != callTransitions.end() && callTransitions.find(key)->second == value)
		return false;

	if(input.is<string::Epsilon>()) {
		for(const auto& transition : callTransitions)
			if(transition.first.first == from )
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");

		for(const auto& transition : returnTransitions)
			if(std::get<0>(transition.first) == from)
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");

		for(const auto& transition : localTransitions)
			if(transition.first.first == from)
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");
	} else {
		for(const auto& transition : callTransitions)
			if(transition.first.first == from && (transition.first.second.is<string::Epsilon>() || transition.first.second == input))
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");

		for(const auto& transition : returnTransitions)
			if(std::get<0>(transition.first) == from && (std::get<1>(transition.first).is<string::Epsilon>() || std::get<1>(transition.first) == input))
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");

		for(const auto& transition : localTransitions)
			if(transition.first.first == from && (transition.first.second.is<string::Epsilon>() || transition.first.second == input))
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");
	}

	callTransitions.insert(std::make_pair(key, value));
	return true;
}

bool RealTimeHeightDeterministicDPDA::addCallTransition(const State& from, const State& to, const alphabet::Symbol& push) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(string::Epsilon::EPSILON);
	return addCallTransition(from, inputVariant, to, push);
}

bool RealTimeHeightDeterministicDPDA::addCallTransition(const State& from, const alphabet::Symbol& input, const State& to, const alphabet::Symbol& push) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(input);
	return addCallTransition(from, inputVariant, to, push);
}

bool RealTimeHeightDeterministicDPDA::addReturnTransition(const State& from, const std::variant<string::Epsilon, alphabet::Symbol>& input, const alphabet::Symbol& pop, const State& to) {
	if (states.find(from) == states.end()) {
		throw AutomatonException("State \"" + (std::string) from.getName() + "\" doesn't exist.");
	}

	if (input.is<alphabet::Symbol>() && inputAlphabet.find(input.get<alphabet::Symbol>()) == inputAlphabet.end()) {
		throw AutomatonException("Input symbol \"" + (std::string) input.get<alphabet::Symbol>() + "\" doesn't exist.");
	}

	if (states.find(to) == states.end()) {
		throw AutomatonException("State \"" + (std::string) to.getName() + "\" doesn't exist.");
	}

	if (stackAlphabet.find(pop) == stackAlphabet.end()) {
		throw AutomatonException("Stack symbol \"" + (std::string) pop + "\" doesn't exist.");
	}

	std::tuple<State, std::variant<string::Epsilon, alphabet::Symbol>, alphabet::Symbol> key(from, input, pop);

	if(returnTransitions.find(key) != returnTransitions.end() && returnTransitions.find(key)->second == to)
		return false;

	if(input.is<string::Epsilon>()) {
		for(const auto& transition : callTransitions)
			if(transition.first.first == from )
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");

		for(const auto& transition : returnTransitions)
			if(std::get<0>(transition.first) == from && std::get<2>(transition.first) == pop)
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");

		for(const auto& transition : localTransitions)
			if(transition.first.first == from)
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");
	} else {
		for(const auto& transition : callTransitions)
			if(transition.first.first == from && (transition.first.second.is<string::Epsilon>() || transition.first.second == input))
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");

		for(const auto& transition : returnTransitions)
			if(std::get<0>(transition.first) == from && (std::get<1>(transition.first).is<string::Epsilon>() || std::get<1>(transition.first) == input) && std::get<2>(transition.first) == pop)
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");

		for(const auto& transition : localTransitions)
			if(transition.first.first == from && (transition.first.second.is<string::Epsilon>() || transition.first.second == input))
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");
	}

	returnTransitions.insert(std::make_pair(key, to));
	return true;
}

bool RealTimeHeightDeterministicDPDA::addReturnTransition(const State& from, const alphabet::Symbol& pop, const State& to) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(string::Epsilon::EPSILON);
	return addReturnTransition(from, inputVariant, pop, to);
}

bool RealTimeHeightDeterministicDPDA::addReturnTransition(const State& from, const alphabet::Symbol& input, const alphabet::Symbol& pop, const State& to) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(input);
	return addReturnTransition(from, inputVariant, pop, to);
}

bool RealTimeHeightDeterministicDPDA::addLocalTransition(const State& from, const std::variant<string::Epsilon, alphabet::Symbol>& input, const State& to) {
	if (states.find(from) == states.end()) {
		throw AutomatonException("State \"" + (std::string) from.getName() + "\" doesn't exist.");
	}

	if (input.is<alphabet::Symbol>() && inputAlphabet.find(input.get<alphabet::Symbol>()) == inputAlphabet.end()) {
		throw AutomatonException("Input symbol \"" + (std::string) input.get<alphabet::Symbol>() + "\" doesn't exist.");
	}

	if (states.find(to) == states.end()) {
		throw AutomatonException("State \"" + (std::string) to.getName() + "\" doesn't exist.");
	}

	std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>> key(from, input);
	
	if(localTransitions.find(key) != localTransitions.end() && localTransitions.find(key)->second == to)
		return false;

	if(input.is<string::Epsilon>()) {
		for(const auto& transition : callTransitions)
			if(transition.first.first == from )
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");

		for(const auto& transition : returnTransitions)
			if(std::get<0>(transition.first) == from)
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");

		for(const auto& transition : localTransitions)
			if(transition.first.first == from)
				throw AutomatonException("Can't add epsilon transition from state \"" + (std::string) from + "\" when other transitions are present.");
	} else {
		for(const auto& transition : callTransitions)
			if(transition.first.first == from && (transition.first.second.is<string::Epsilon>() || transition.first.second == input))
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");

		for(const auto& transition : returnTransitions)
			if(std::get<0>(transition.first) == from && (std::get<1>(transition.first).is<string::Epsilon>() || std::get<1>(transition.first) == input))
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");

		for(const auto& transition : localTransitions)
			if(transition.first.first == from && (transition.first.second.is<string::Epsilon>() || transition.first.second == input))
				throw AutomatonException("Can't add transition from state \"" + (std::string) from + "\" when transition reading \"" + (std::string) input + "\" is present.");
	}

	localTransitions.insert(std::make_pair(key, to));
	return true;
}

bool RealTimeHeightDeterministicDPDA::addLocalTransition(const State& from, const State& to) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(string::Epsilon::EPSILON);
	return addLocalTransition(from, inputVariant, to);
}

bool RealTimeHeightDeterministicDPDA::addLocalTransition(const State& from, const alphabet::Symbol& input, const State& to) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(input);
	return addLocalTransition(from, inputVariant, to);
}

bool RealTimeHeightDeterministicDPDA::removeCallTransition(const State& from, const std::variant<string::Epsilon, alphabet::Symbol>& input, const State& to, const alphabet::Symbol& push) {
	std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>> key(from, input);
	std::pair<automaton::State, alphabet::Symbol> value = std::make_pair(to, push);

	if (callTransitions.find(key) == callTransitions.end())
		return false;

	if(callTransitions.find(key)->second != value)
		throw AutomatonException(
				"Transition (\"" + (std::string) from.getName() + "\", \"" + (std::string) input
						+ "\") -> \"" + (std::string) to.getName() + "\" doesn't exist.");

	callTransitions.erase(key);
	return true;
}

bool RealTimeHeightDeterministicDPDA::removeCallTransition(const State& from, const State& to, const alphabet::Symbol& push) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(string::Epsilon::EPSILON);
	return removeCallTransition(from, inputVariant, to, push);
}

bool RealTimeHeightDeterministicDPDA::removeCallTransition(const State& from, const alphabet::Symbol& input, const State& to, const alphabet::Symbol& push) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(input);
	return removeCallTransition(from, inputVariant, to, push);
}

bool RealTimeHeightDeterministicDPDA::removeReturnTransition(const State& from, const std::variant<string::Epsilon, alphabet::Symbol>& input, const alphabet::Symbol& pop, const State& to) {
	std::tuple<State, std::variant<string::Epsilon, alphabet::Symbol>, alphabet::Symbol> key(from, input, pop);

	if (returnTransitions.find(key) == returnTransitions.end())
		return false;

	if(returnTransitions.find(key)->second != to)
		throw AutomatonException(
				"Transition (\"" + (std::string) from.getName() + "\", \"" + (std::string) input
						+ "\") -> \"" + (std::string) to.getName() + "\" doesn't exist.");

	returnTransitions.erase(key);
	return true;
}

bool RealTimeHeightDeterministicDPDA::removeReturnTransition(const State& from, const alphabet::Symbol& pop, const State& to) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(string::Epsilon::EPSILON);
	return removeReturnTransition(from, inputVariant, pop, to);
}

bool RealTimeHeightDeterministicDPDA::removeReturnTransition(const State& from, const alphabet::Symbol& input, const alphabet::Symbol& pop, const State& to) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(input);
	return removeReturnTransition(from, inputVariant, pop, to);
}

bool RealTimeHeightDeterministicDPDA::removeLocalTransition(const State& from, const std::variant<string::Epsilon, alphabet::Symbol>& input, const State& to) {
	std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>> key(from, input);

	if (localTransitions.find(key) == localTransitions.end())
		return false;

	if(localTransitions.find(key)->second != to)
		throw AutomatonException(
				"Transition (\"" + (std::string) from.getName() + "\", \"" + (std::string) input
						+ "\") -> \"" + (std::string) to.getName() + "\" doesn't exist.");

	localTransitions.erase(key);
	return true;
}

bool RealTimeHeightDeterministicDPDA::removeLocalTransition(const State& from, const State& to) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(string::Epsilon::EPSILON);
	return removeLocalTransition(from, inputVariant, to);
}

bool RealTimeHeightDeterministicDPDA::removeLocalTransition(const State& from, const alphabet::Symbol& input, const State& to) {
	std::variant<string::Epsilon, alphabet::Symbol> inputVariant(input);
	return removeLocalTransition(from, inputVariant, to);
}

const std::map<std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>>, std::pair<State, alphabet::Symbol> >& RealTimeHeightDeterministicDPDA::getCallTransitions() const {
		return callTransitions;
}

const std::map<std::tuple<State, std::variant<string::Epsilon, alphabet::Symbol>, alphabet::Symbol>, State>& RealTimeHeightDeterministicDPDA::getReturnTransitions() const {
		return returnTransitions;
}

const std::map<std::pair<State, std::variant<string::Epsilon, alphabet::Symbol>>, State>& RealTimeHeightDeterministicDPDA::getLocalTransitions() const {
		return localTransitions;
}

int RealTimeHeightDeterministicDPDA::compare(const RealTimeHeightDeterministicDPDA& other) const {
	auto first = std::tie(states, inputAlphabet, initialState, finalStates, stackAlphabet, bottomOfTheStackSymbol, callTransitions, returnTransitions, localTransitions);
	auto second = std::tie(other.states, other.inputAlphabet, other.initialState, other.finalStates, other.stackAlphabet, other.bottomOfTheStackSymbol, other.callTransitions, other.returnTransitions, other.localTransitions);

	std::compare<decltype(first)> comp;
	return comp(first, second);
}

void RealTimeHeightDeterministicDPDA::operator>>(std::ostream& out) const {
	out << "(RealTimeHeightDeterministicDPDA "
		<< "states = " << states
		<< "inputAlphabet = " << inputAlphabet
		<< "initialState = " << initialState
		<< "finalStates = " << finalStates
		<< "stackAlphabet = " << stackAlphabet
		<< "bottomOfTheStackSymbol = " << bottomOfTheStackSymbol
		<< "callTransitions = " << callTransitions
		<< "returnTransitions = " << returnTransitions
		<< "localTransitions = " << localTransitions
		<< ")";
}

RealTimeHeightDeterministicDPDA::operator std::string () const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace automaton */
