/*
 * ContainerToXMLComposer.h
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#ifndef CONTAINER_TO_XML_COMPOSER_H_
#define CONTAINER_TO_XML_COMPOSER_H_

#include <list>
#include "../std/variant.hpp"
#include "../sax/Token.h"

#include "Container.h"
#include "ObjectsSet.h"
#include "ObjectsVector.h"
#include "ObjectsPair.h"

namespace alib {

template<typename T>
struct xmlApi;

} /* namespace alib */

namespace container {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class ContainerToXMLComposer {
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	void compose(std::list<sax::Token>& output, const ContainerBase& container) const;

	/**
	 * Prints XML representation of String to the output stream.
	 * @param string String to print
	 * @param out output stream to which print the String
	 */
	void compose(std::list<sax::Token>& output, const Container& container) const;

	template<typename T>
	void compose(std::list<sax::Token>& output, const std::set<T>& container) const;

	void compose(std::list<sax::Token>& output, const ObjectsSet& container) const;

	template<typename T>
	void compose(std::list<sax::Token>& output, const std::vector<T>& container) const;

	void compose(std::list<sax::Token>& output, const ObjectsVector& container) const;

	template<typename T, typename R>
	void compose(std::list<sax::Token>& output, const std::pair<T, R>& container) const;

	void compose(std::list<sax::Token>& output, const ObjectsPair& container) const;

	template<typename T, typename R>
	void compose(std::list<sax::Token>& output, const std::map<T, R>& container) const;

	void compose(std::list<sax::Token>& output, const ObjectsMap& container) const;

	template<typename ... Ts>
	void compose(std::list<sax::Token>& output, const std::variant<Ts...>& container) const;

	template<typename T> friend class alib::xmlApi;
};

} /* namespace container */

#include "../XmlApi.hpp"

namespace container {

template<typename T>
void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const std::set<T>& container) const {
	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_SET, sax::Token::TokenType::START_ELEMENT));

	for(const T& item : container) {
		alib::xmlApi<T>::compose(out, item);
	}

	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_SET, sax::Token::TokenType::END_ELEMENT));
}

template<typename T>
void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const std::vector<T>& container) const {
	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_VECTOR, sax::Token::TokenType::START_ELEMENT));

	for(const T& item : container) {
		alib::xmlApi<T>::compose(out, item);
	}

	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_VECTOR, sax::Token::TokenType::END_ELEMENT));
}

template<typename T, typename R>
void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const std::pair<T, R>& container) const {
	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_PAIR, sax::Token::TokenType::START_ELEMENT));

	alib::xmlApi<T>::compose(out, container.first);
	alib::xmlApi<R>::compose(out, container.second);

	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_PAIR, sax::Token::TokenType::END_ELEMENT));
}

template<typename T, typename R>
void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const std::map<T, R>& container) const {
	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_MAP, sax::Token::TokenType::START_ELEMENT));

	for(const std::pair<T, R>& item : container) {
		alib::xmlApi<std::pair<T, R>>::compose(out, item);
	}

	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_MAP, sax::Token::TokenType::END_ELEMENT));
}

/* TODO for gcc-4.9 uncomment
template<typename T>
void composeVariantHelper(const T&) {
	throw exception::AlibException("Can't compose variant type");
}

template<typename T, typename R, typename ... Ts>
void composeVariantHelper(std::list<sax::Token>& out, const T& container) {
	if(container.template is<R>()) return alib::xmlApi<R>::compose(out, container.template get<R>());
	composeVariantHelper<T, Ts...>(out, container);
}

template<typename ... Ts>
void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const std::variant<Ts...>& container) const {
	composeVariantHelper<std::variant<Ts...>, Ts...>(out, container);
}
*/

} /* namespace container */

#endif /* CONTAINER_TO_XML_COMPOSER_H_ */

