/*
 * EpsilonFreeCFG.cpp
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#include "EpsilonFreeCFG.h"
#include "../../std/map.hpp"
#include "../GrammarException.h"
#include <algorithm>

#include "../../alphabet/Symbol.h"

namespace grammar {

EpsilonFreeCFG::EpsilonFreeCFG(const alphabet::Symbol& initialSymbol) : TerminalNonterminalAlphabetInitialSymbol(initialSymbol), generatesEpsilon(false) {

}

EpsilonFreeCFG::EpsilonFreeCFG(const std::set<alphabet::Symbol>& nonterminalAlphabet, const std::set<alphabet::Symbol>& terminalAlphabet, const alphabet::Symbol& initialSymbol) : TerminalNonterminalAlphabetInitialSymbol(initialSymbol), generatesEpsilon(false) {
	setNonterminalAlphabet(nonterminalAlphabet);
	setTerminalAlphabet(terminalAlphabet);
}

GrammarBase* EpsilonFreeCFG::clone() const {
	return new EpsilonFreeCFG(*this);
}

GrammarBase* EpsilonFreeCFG::plunder() && {
	return new EpsilonFreeCFG(std::move(*this));
}

bool EpsilonFreeCFG::removeTerminalSymbol(const alphabet::Symbol& symbol) {
	for(const std::pair<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>>& rule : rules) {
		for(const std::vector<alphabet::Symbol>& rhs : rule.second)
			if(std::find(rhs.begin(), rhs.end(), symbol) != rhs.end())
				throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");
	}

	return terminalAlphabet.erase(symbol);
}

bool EpsilonFreeCFG::removeNonterminalSymbol(const alphabet::Symbol& symbol) {
	for(const std::pair<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>>& rule : rules) {
		if(rule.first == symbol)
			throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");

		for(const std::vector<alphabet::Symbol>& rhs : rule.second)
			if(std::find(rhs.begin(), rhs.end(), symbol) != rhs.end())
				throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");
	}

	if(initialSymbol == symbol)
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" is initial symbol.");


	return nonterminalAlphabet.erase(symbol);
}

bool EpsilonFreeCFG::addRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	int rSize = rightHandSide.size();

	if(rSize == 0) {
		throw GrammarException("Epsilon rule is not allowed");
	} else {
		if(!nonterminalAlphabet.count(leftHandSide))
			throw GrammarException("Rule must rewrite nonterminal symbol");

		for(const alphabet::Symbol& symbol : rightHandSide) {
			if(terminalAlphabet.find(symbol) == terminalAlphabet.end() && nonterminalAlphabet.find(symbol) == nonterminalAlphabet.end())
				throw GrammarException("Symbol \"" + (std::string) symbol + "\" is not neither terminal nor nonterminal symbol");
		}

		return rules[leftHandSide].insert(rightHandSide).second;
	}
}

const std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> & EpsilonFreeCFG::getRules() const {
	return rules;
}

bool EpsilonFreeCFG::removeRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	return rules[leftHandSide].erase(rightHandSide);
}

bool EpsilonFreeCFG::addRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	if(rightHandSide.size() == 0) {
		if(leftHandSide != initialSymbol) throw exception::AlibException("Illegal left hand side of epsilon rule");
		bool res = getGeneratesEpsilon();
		setGeneratesEpsilon(true);
		return !res;
	} else {
		return addRule(leftHandSide, rightHandSide);
	}
}

std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> EpsilonFreeCFG::getRawRules() const {
	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> res(getRules().begin(), getRules().end());
	if(getGeneratesEpsilon()) {
		res[getInitialSymbol()].insert(std::vector<alphabet::Symbol> {});
	}
	return rules;
}

bool EpsilonFreeCFG::removeRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	if(rightHandSide.size() == 0) {
		if(leftHandSide != initialSymbol) throw exception::AlibException("Illegal left hand side of epsilon rule");
		bool res = getGeneratesEpsilon();
		setGeneratesEpsilon(false);
		return res;
	} else {
		return removeRule(leftHandSide, rightHandSide);
	}
}

void EpsilonFreeCFG::setGeneratesEpsilon(bool genEps) {
	generatesEpsilon = genEps;
}

bool EpsilonFreeCFG::getGeneratesEpsilon() const {
	return generatesEpsilon;
}

int EpsilonFreeCFG::compare(const EpsilonFreeCFG& other) const {
	auto first = std::tie(terminalAlphabet, nonterminalAlphabet, initialSymbol, rules);
	auto second = std::tie(other.terminalAlphabet, other.nonterminalAlphabet, other.initialSymbol, other.rules);

	std::compare<decltype(first)> comp;
	return comp(first, second);
}

void EpsilonFreeCFG::operator>>(std::ostream& out) const {
	out << "(EpsilonFreeCFG "
			<< "nonterminalAlphabet = " << nonterminalAlphabet
			<< "terminalAlphabet = " << terminalAlphabet
			<< "initialSymbol = " << initialSymbol
			<< "rules = " << rules
			<< "generatesEpsilon = " << generatesEpsilon
			<< ")";
}

EpsilonFreeCFG::operator std::string () const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace grammar */
