/*
 * LG.cpp
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#include "LG.h"
#include "../../std/map.hpp"
#include "../GrammarException.h"
#include <algorithm>

#include "../../alphabet/Symbol.h"

namespace grammar {

LG::LG(const alphabet::Symbol& initialSymbol) : TerminalNonterminalAlphabetInitialSymbol(initialSymbol) {

}

LG::LG(const std::set<alphabet::Symbol>& nonterminalAlphabet, const std::set<alphabet::Symbol>& terminalAlphabet, const alphabet::Symbol& initialSymbol) : TerminalNonterminalAlphabetInitialSymbol(initialSymbol) {
	setNonterminalAlphabet(nonterminalAlphabet);
	setTerminalAlphabet(terminalAlphabet);
}

GrammarBase* LG::clone() const {
	return new LG(*this);
}

GrammarBase* LG::plunder() && {
	return new LG(std::move(*this));
}

bool LG::removeTerminalSymbol(const alphabet::Symbol& symbol) {
	for(const std::pair<alphabet::Symbol, std::set<std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> >> >& rule : rules) {
		for(const std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> >& rhsTmp : rule.second)
			if(rhsTmp.is<std::vector<alphabet::Symbol>>()) {
				const std::vector<alphabet::Symbol>& rhs = rhsTmp.get<std::vector<alphabet::Symbol>>();

				if(std::find(rhs.begin(), rhs.end(), symbol) != rhs.end())
					throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");
			} else {
				const std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>& rhs = rhsTmp.get<std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>();

				const std::vector<alphabet::Symbol>& lPart = std::get<0>(rhs);
				if(std::find(lPart.begin(), lPart.end(), symbol) != lPart.end())
					throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");
				
				const std::vector<alphabet::Symbol>& rPart = std::get<2>(rhs);
				if(std::find(rPart.begin(), rPart.end(), symbol) != rPart.end())
					throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");
			}
	}

	return terminalAlphabet.erase(symbol);
}

bool LG::removeNonterminalSymbol(const alphabet::Symbol& symbol) {
	for(const std::pair<alphabet::Symbol, std::set<std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> >> >& rule : rules) {
		if(rule.first == symbol)
			throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");

		for(const std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> >& rhsTmp : rule.second)
			if(rhsTmp.is<std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>()) {
				const std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>& rhs = rhsTmp.get<std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>();

				if(std::get<1>(rhs) == symbol)
					throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");
			}
	}

	if(initialSymbol == symbol)
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" is initial symbol.");


	return nonterminalAlphabet.erase(symbol);
}

bool LG::addRule(const alphabet::Symbol& leftHandSide, const std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> >& rightHandSide) {
	if(!nonterminalAlphabet.count(leftHandSide))
		throw GrammarException("Rule must rewrite nonterminal symbol");

	if(rightHandSide.is<std::vector<alphabet::Symbol>>()) {
		const std::vector<alphabet::Symbol>& rhs = rightHandSide.get<std::vector<alphabet::Symbol>>();
		for(const auto & symbol : rhs ) {
			if(terminalAlphabet.find(symbol) == terminalAlphabet.end())
				throw GrammarException("Symbol " + (std::string) symbol + " is not a terminal symbol");
		}

		return rules[leftHandSide].insert(rightHandSide).second;
	} else {
		const std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>& rhs = rightHandSide.get<std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>();

		for(const auto & symbol : std::get<0>(rhs) )
			if(terminalAlphabet.find(symbol) == terminalAlphabet.end())
				throw GrammarException("Symbol " + (std::string) symbol + " is not a terminal symbol");

		if(nonterminalAlphabet.find(std::get<1>(rhs)) == nonterminalAlphabet.end())
			throw GrammarException("Symbol " + (std::string) std::get<1>(rhs) + " is not a nonterminal symbol");

		for(const auto & symbol : std::get<2>(rhs) ) {
			if(terminalAlphabet.find(symbol) == terminalAlphabet.end())
				throw GrammarException("Symbol " + (std::string) symbol + " is not a terminal symbol");
		}

		return rules[leftHandSide].insert(rightHandSide).second;
	}
}

bool LG::addRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> > rhs(rightHandSide);
	return addRule(leftHandSide, rhs);
}

bool LG::addRule(const alphabet::Symbol& leftHandSide, const std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>& rightHandSide) {
	std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> > rhs(rightHandSide);
	return addRule(leftHandSide, rhs);
}

const std::map<alphabet::Symbol, std::set<std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> >> > & LG::getRules() const {
	return rules;
}

bool LG::removeRule(const alphabet::Symbol& leftHandSide, const std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> >& rightHandSide) {
	return rules[leftHandSide].erase(rightHandSide);
}

bool LG::removeRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> > rhs(rightHandSide);
	return removeRule(leftHandSide, rhs);
}

bool LG::removeRule(const alphabet::Symbol& leftHandSide, const std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>& rightHandSide) {
	std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>> > rhs(rightHandSide);
	return removeRule(leftHandSide, rhs);
}

bool LG::addRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	std::vector<alphabet::Symbol>::const_iterator nonterminalPosition = rightHandSide.begin();
	for(; nonterminalPosition != rightHandSide.end(); nonterminalPosition++) {
		if(nonterminalAlphabet.count(*nonterminalPosition)) break;
	}

	if(nonterminalPosition == rightHandSide.end()) {
		return addRule(leftHandSide, rightHandSide);
	} else {
		return addRule(leftHandSide, std::make_tuple(std::vector<alphabet::Symbol>(rightHandSide.begin(), nonterminalPosition), *nonterminalPosition, std::vector<alphabet::Symbol>(nonterminalPosition + 1, rightHandSide.end())));
	}
}

std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> LG::getRawRules() const {
	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> res;
	for(const auto& rule : getRules()) {
		for(const auto& rhs : rule.second) {
			if(rhs.is<std::vector<alphabet::Symbol>>()) {
				res[rule.first].insert(rhs.get<std::vector<alphabet::Symbol>>());
			} else {
				const auto& rhsTuple = rhs.get<std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>();
				std::vector<alphabet::Symbol> tmp;
				tmp.insert(tmp.end(), std::get<0>(rhsTuple).begin(), std::get<0>(rhsTuple).end());
				tmp.push_back(std::get<1>(rhsTuple));
				tmp.insert(tmp.end(), std::get<2>(rhsTuple).begin(), std::get<2>(rhsTuple).end());
				res[rule.first].insert(tmp);
			}
		}
	}
	return res;
}

bool LG::removeRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	std::vector<alphabet::Symbol>::const_iterator nonterminalPosition = rightHandSide.begin();
	for(; nonterminalPosition != rightHandSide.end(); nonterminalPosition++) {
		if(nonterminalAlphabet.count(*nonterminalPosition)) break;
	}

	if(nonterminalPosition == rightHandSide.end()) {
		return removeRule(leftHandSide, rightHandSide);
	} else {
		return removeRule(leftHandSide, std::make_tuple(std::vector<alphabet::Symbol>(rightHandSide.begin(), nonterminalPosition), *nonterminalPosition, std::vector<alphabet::Symbol>(nonterminalPosition + 1, rightHandSide.end())));
	}
}

int LG::compare(const LG& other) const {
	auto first = std::tie(terminalAlphabet, nonterminalAlphabet, initialSymbol, rules);
	auto second = std::tie(other.terminalAlphabet, other.nonterminalAlphabet, other.initialSymbol, other.rules);

	std::compare<decltype(first)> comp;
	return comp(first, second);
}

void LG::operator>>(std::ostream& out) const {
	out << "(LG "
			<< "nonterminalAlphabet = " << nonterminalAlphabet
			<< "terminalAlphabet = " << terminalAlphabet
			<< "initialSymbol = " << initialSymbol
			<< "rules = " << rules
			<< ")";
}

LG::operator std::string () const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace grammar */
