/*
 * NonContractingGrammar.cpp
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#include "NonContractingGrammar.h"
#include "../../std/map.hpp"
#include "../GrammarException.h"
#include <algorithm>
#include <sstream>

#include "../../alphabet/Symbol.h"

namespace grammar {

NonContractingGrammar::NonContractingGrammar(const alphabet::Symbol& initialSymbol) : TerminalNonterminalAlphabetInitialSymbol(initialSymbol), generatesEpsilon(false) {

}

NonContractingGrammar::NonContractingGrammar(const std::set<alphabet::Symbol>& nonterminalAlphabet, const std::set<alphabet::Symbol>& terminalAlphabet, const alphabet::Symbol& initialSymbol) : TerminalNonterminalAlphabetInitialSymbol(initialSymbol), generatesEpsilon(false) {
	setNonterminalAlphabet(nonterminalAlphabet);
	setTerminalAlphabet(terminalAlphabet);
}

GrammarBase* NonContractingGrammar::clone() const {
	return new NonContractingGrammar(*this);
}

GrammarBase* NonContractingGrammar::plunder() && {
	return new NonContractingGrammar(std::move(*this));
}

bool NonContractingGrammar::removeTerminalSymbol(const alphabet::Symbol& symbol) {
	for(const std::pair<std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol>>>& rule : rules) {
		if(std::find(rule.first.begin(), rule.first.end(), symbol) != rule.first.end())
			throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");

		for(const std::vector<alphabet::Symbol>& rhs : rule.second)
			if(std::find(rhs.begin(), rhs.end(), symbol) != rhs.end())
				throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");
	}

	return terminalAlphabet.erase(symbol);
}

bool NonContractingGrammar::removeNonterminalSymbol(const alphabet::Symbol& symbol) {
	for(const std::pair<std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol>>>& rule : rules) {
		if(std::find(rule.first.begin(), rule.first.end(), symbol) != rule.first.end())
			throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");

		for(const std::vector<alphabet::Symbol>& rhs : rule.second)
			if(std::find(rhs.begin(), rhs.end(), symbol) != rhs.end())
				throw GrammarException("Symbol \"" + (std::string) symbol + "\" is used in rule.");
	}

	if(initialSymbol == symbol)
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" is initial symbol.");


	return nonterminalAlphabet.erase(symbol);
}

bool NonContractingGrammar::addRule(const std::vector<alphabet::Symbol>& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	int lSize = leftHandSide.size();
	int rSize = rightHandSide.size();

	if(lSize > rSize)
		throw GrammarException("Invalid size of right hand side of a rule");

	if(std::all_of(leftHandSide.begin(), leftHandSide.end(), [&](const alphabet::Symbol symbol) {return !nonterminalAlphabet.count(symbol);}))
		throw GrammarException("Rule must rewrite nonterminal symbol");

	for(const alphabet::Symbol& symbol : leftHandSide)
		if(terminalAlphabet.find(symbol) == terminalAlphabet.end() && nonterminalAlphabet.find(symbol) == nonterminalAlphabet.end())
			throw GrammarException("Symbol \"" + (std::string) symbol + "\" is not neither terminal nor nonterminal symbol");

	for(const alphabet::Symbol& symbol : rightHandSide) {
		if(terminalAlphabet.find(symbol) == terminalAlphabet.end() && nonterminalAlphabet.find(symbol) == nonterminalAlphabet.end())
			throw GrammarException("Symbol \"" + (std::string) symbol + "\" is not neither terminal nor nonterminal symbol");
	}

	return rules[leftHandSide].insert(rightHandSide).second;
}

const std::map<std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol>>> & NonContractingGrammar::getRules() const {
	return rules;
}

bool NonContractingGrammar::removeRule(const std::vector<alphabet::Symbol>& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide) {
	return rules[leftHandSide].erase(rightHandSide);
}

void NonContractingGrammar::setGeneratesEpsilon(bool genEps) {
	generatesEpsilon = genEps;
}

bool NonContractingGrammar::getGeneratesEpsilon() const {
	return generatesEpsilon;
}

int NonContractingGrammar::compare(const NonContractingGrammar& other) const {
	auto first = std::tie(terminalAlphabet, nonterminalAlphabet, initialSymbol, rules);
	auto second = std::tie(other.terminalAlphabet, other.nonterminalAlphabet, other.initialSymbol, other.rules);

	std::compare<decltype(first)> comp;
	return comp(first, second);
}

void NonContractingGrammar::operator>>(std::ostream& out) const {
	out << "(NonContractingGrammar "
			<< "nonterminalAlphabet = " << nonterminalAlphabet
			<< "terminalAlphabet = " << terminalAlphabet
			<< "initialSymbol = " << initialSymbol
			<< "rules = " << rules
			<< "generatesEpsilon = " << generatesEpsilon
			<< ")";
}

NonContractingGrammar::operator std::string () const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace grammar */
