/*
 * GrammarFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarFromStringParser.h"

#include "../exception/AlibException.h"

#include "Regular/RightRG.h"
#include "ContextFree/CNF.h"

#include "../StringApi.hpp"

namespace grammar {

Grammar GrammarFromStringParser::parseGrammar(std::istream& input) const {
	return parseGrammar(input, std::set<FEATURES>({FEATURES::CNF, FEATURES::RIGHT_RG}));
}

Grammar GrammarFromStringParser::parseGrammar(std::istream& input, const std::set<FEATURES>& features) const {
	GrammarFromStringLexer::Token token = m_GrammarLexer.next(input);
	if(token.type == GrammarFromStringLexer::TokenType::CNF) {
		if(!features.count(FEATURES::CNF)) throw exception::AlibException("Disabled formalism CNF");
		m_GrammarLexer.putback(input, token);
		return Grammar(parseCNF(input));
	} else if(token.type == GrammarFromStringLexer::TokenType::RIGHT_RG) {
		if(!features.count(FEATURES::RIGHT_RG)) throw exception::AlibException("Disabled Formalism RightRG");
		m_GrammarLexer.putback(input, token);
		return Grammar(parseRightRG(input));
	} else {
		throw exception::AlibException("Formalism not recognised (token = \"" + token.value + "\")");
	}
}

std::set<alphabet::Symbol> GrammarFromStringParser::parseSet(std::istream& input) const {
	std::set<alphabet::Symbol> res;

	GrammarFromStringLexer::Token token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::SET_BEGIN) {
		throw exception::AlibException();
	}

	token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::SET_END) {
		m_GrammarLexer.putback(input, token);
		while(true) {
			alphabet::Symbol symbol = alib::stringApi<alphabet::Symbol>::parse(input);
			res.insert(symbol);

			token = m_GrammarLexer.next(input);
			if(token.type == GrammarFromStringLexer::TokenType::SET_END) {
				break;
			}
			if(token.type != GrammarFromStringLexer::TokenType::COMMA) {
				throw exception::AlibException("Expected SET_END or COMMA token");
			}
		}
	}

	if(token.type != GrammarFromStringLexer::TokenType::SET_END) {
		throw exception::AlibException("Expected SET_END token");
	}

	return res;
}

std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> GrammarFromStringParser::parseCFLikeRules(std::istream& input) const {
	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> result;

	GrammarFromStringLexer::Token token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::SET_BEGIN) {
		throw exception::AlibException();
	}

	token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::SET_END) {
		m_GrammarLexer.putback(input, token);
		while(true) {
			alphabet::Symbol lhs = alib::stringApi<alphabet::Symbol>::parse(input);

			token = m_GrammarLexer.next(input);
			if(token.type != GrammarFromStringLexer::TokenType::MAPS_TO) {
				throw exception::AlibException();
			}

			while(true) {
				std::vector<alphabet::Symbol> rhs;

				token = m_GrammarLexer.next(input);
				if(token.type != GrammarFromStringLexer::TokenType::COMMA && token.type != GrammarFromStringLexer::TokenType::SET_END && token.type != GrammarFromStringLexer::TokenType::SEPARATOR) while(true) {
					m_GrammarLexer.putback(input, token);

					rhs.push_back(alib::stringApi<alphabet::Symbol>::parse(input));
					token = m_GrammarLexer.next(input);
					if(token.type == GrammarFromStringLexer::TokenType::SEPARATOR || token.type == GrammarFromStringLexer::TokenType::COMMA || token.type == GrammarFromStringLexer::TokenType::SET_END) {
						break;
					}
				}
				result[lhs].insert(rhs);
				if(token.type == GrammarFromStringLexer::TokenType::COMMA || token.type == GrammarFromStringLexer::TokenType::SET_END) {
					break;
				}
				if(token.type != GrammarFromStringLexer::TokenType::SEPARATOR) {
					throw exception::AlibException("Expected SEPARATOR, SETEND or COMMA token");
				}
			}

			if(token.type == GrammarFromStringLexer::TokenType::SET_END) {
				break;
			}
			if(token.type != GrammarFromStringLexer::TokenType::COMMA) {
				throw exception::AlibException("Expected SET_END or COMMA token");
			}
		}
	}

	if(token.type != GrammarFromStringLexer::TokenType::SET_END) {
		throw exception::AlibException("Expected SET_END token");
	}
	return result;
}

RightRG GrammarFromStringParser::parseRightRG(std::istream& input) const {
	throw exception::AlibException("Unimplemented");
}

CNF GrammarFromStringParser::parseCNF(std::istream& input) const {
	GrammarFromStringLexer::Token token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::CNF) {
		throw exception::AlibException("Unrecognised CNF token.");
	}

	token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::TUPLE_BEGIN) {
		throw exception::AlibException("Unrecognised Tuple begin token.");
	}

	std::set<alphabet::Symbol> nonterminals = parseSet(input);

	token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::COMMA) {
		throw exception::AlibException("Unrecognised Comma token.");
	}

	std::set<alphabet::Symbol> terminals = parseSet(input);

	token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::COMMA) {
		throw exception::AlibException("Unrecognised Comma token.");
	}

	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> rules = parseCFLikeRules(input);

	token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::COMMA) {
		throw exception::AlibException("Unrecognised Comma token.");
	}

	alphabet::Symbol initialSymbol = alib::stringApi<alphabet::Symbol>::parse(input);

	token = m_GrammarLexer.next(input);
	if(token.type != GrammarFromStringLexer::TokenType::TUPLE_END) {
		throw exception::AlibException("Unrecognised Tuple end token.");
	}

	CNF cnf(nonterminals, terminals, initialSymbol);
	for(const auto& rule : rules) {
		for(const auto& ruleRHS : rule.second) {
			cnf.addRawRule(rule.first, ruleRHS);
		}
	}
	return cnf;
}

} /* namespace grammar */
