/*
 * GrammarFromXMLParser.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarFromXMLParser.h"

#include "../sax/ParserException.h"
#include "../alphabet/LabeledSymbol.h"

#include "../XmlApi.hpp"

namespace grammar {

Grammar GrammarFromXMLParser::parseGrammar(std::list<sax::Token> &input) const {
	return parseGrammar(input, std::set<FEATURES>({FEATURES::LEFT_LG, FEATURES::LEFT_RG, FEATURES::RIGHT_LG, FEATURES::RIGHT_RG, FEATURES::LG, FEATURES::CFG, FEATURES::EPSILON_FREE_CFG, FEATURES::CNF, FEATURES::GNF, FEATURES::CSG, FEATURES::NON_CONTRACTING_GRAMMAR, FEATURES::CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR, FEATURES::UNRESTRICTED_GRAMMAR}));
}

Grammar GrammarFromXMLParser::parseGrammar(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(alib::xmlApi<UnrestrictedGrammar>::first(input)) {
		if(!features.count(FEATURES::UNRESTRICTED_GRAMMAR)) throw exception::AlibException();
		return Grammar(parseUnrestrictedGrammar(input));
	} else if(alib::xmlApi<ContextPreservingUnrestrictedGrammar>::first(input)) {
		if(!features.count(FEATURES::CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR)) throw exception::AlibException();
		return Grammar(parseContextPreservingUnrestrictedGrammar(input));
	} else if(alib::xmlApi<NonContractingGrammar>::first(input)) {
		if(!features.count(FEATURES::NON_CONTRACTING_GRAMMAR)) throw exception::AlibException();
		return Grammar(parseNonContractingGrammar(input));
	} else if(alib::xmlApi<CSG>::first(input)) {
		if(!features.count(FEATURES::CSG)) throw exception::AlibException();
		return Grammar(parseCSG(input));
	} else if(alib::xmlApi<GNF>::first(input)) {
		if(!features.count(FEATURES::GNF)) throw exception::AlibException();
		return Grammar(parseGNF(input));
	} else if(alib::xmlApi<CNF>::first(input)) {
		if(!features.count(FEATURES::CNF)) throw exception::AlibException();
		return Grammar(parseCNF(input));
	} else if(alib::xmlApi<EpsilonFreeCFG>::first(input)) {
		if(!features.count(FEATURES::EPSILON_FREE_CFG)) throw exception::AlibException();
		return Grammar(parseEpsilonFreeCFG(input));
	} else if(alib::xmlApi<CFG>::first(input)) {
		if(!features.count(FEATURES::CFG)) throw exception::AlibException();
		return Grammar(parseCFG(input));
	} else if(alib::xmlApi<LG>::first(input)) {
		if(!features.count(FEATURES::LG)) throw exception::AlibException();
		return Grammar(parseLG(input));
	} else if(alib::xmlApi<RightRG>::first(input)) {
		if(!features.count(FEATURES::RIGHT_RG)) throw exception::AlibException();
		return Grammar(parseRightRG(input));
	} else if(alib::xmlApi<RightLG>::first(input)) {
		if(!features.count(FEATURES::RIGHT_LG)) throw exception::AlibException();
		return Grammar(parseRightLG(input));
	} else if(alib::xmlApi<LeftRG>::first(input)) {
		if(!features.count(FEATURES::LEFT_RG)) throw exception::AlibException();
		return Grammar(parseLeftRG(input));
	} else if(alib::xmlApi<LeftLG>::first(input)) {
		if(!features.count(FEATURES::LEFT_LG)) throw exception::AlibException();
		return Grammar(parseLeftLG(input));
	} else
		throw sax::ParserException(sax::Token("Grammar / UnrestrictedGrammar", sax::Token::TokenType::START_ELEMENT), input.front());
}

bool GrammarFromXMLParser::first(const std::list<sax::Token>& input) const {
	if(alib::xmlApi<UnrestrictedGrammar>::first(input) || alib::xmlApi<ContextPreservingUnrestrictedGrammar>::first(input) || alib::xmlApi<NonContractingGrammar>::first(input) || alib::xmlApi<CSG>::first(input) || alib::xmlApi<GNF>::first(input) || alib::xmlApi<CNF>::first(input) || alib::xmlApi<EpsilonFreeCFG>::first(input) || alib::xmlApi<CFG>::first(input) || alib::xmlApi<LG>::first(input) || alib::xmlApi<RightRG>::first(input) || alib::xmlApi<RightLG>::first(input) || alib::xmlApi<LeftLG>::first(input) || alib::xmlApi<LeftRG>::first(input)) {
		return true;
	} else {
		return false;
	}
}

UnrestrictedGrammar GrammarFromXMLParser::parseUnrestrictedGrammar(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_UNRESTRICTED_GRAMMAR);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	UnrestrictedGrammar grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_UNRESTRICTED_GRAMMAR);
	return grammar;
}

ContextPreservingUnrestrictedGrammar GrammarFromXMLParser::parseContextPreservingUnrestrictedGrammar(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	ContextPreservingUnrestrictedGrammar grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR);
	return grammar;
}

NonContractingGrammar GrammarFromXMLParser::parseNonContractingGrammar(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_NON_CONTRACTING_GRAMMAR);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	NonContractingGrammar grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	bool generatesEpsilon = parseGeneratesEpsilon(input);
	grammar.setGeneratesEpsilon(generatesEpsilon);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_NON_CONTRACTING_GRAMMAR);
	return grammar;
}

CSG GrammarFromXMLParser::parseCSG(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_CSG);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	CSG grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	bool generatesEpsilon = parseGeneratesEpsilon(input);
	grammar.setGeneratesEpsilon(generatesEpsilon);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_CSG);
	return grammar;
}

GNF GrammarFromXMLParser::parseGNF(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_GNF);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	GNF grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	bool generatesEpsilon = parseGeneratesEpsilon(input);
	grammar.setGeneratesEpsilon(generatesEpsilon);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_GNF);
	return grammar;
}

CNF GrammarFromXMLParser::parseCNF(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_CNF);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	CNF grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	bool generatesEpsilon = parseGeneratesEpsilon(input);
	grammar.setGeneratesEpsilon(generatesEpsilon);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_CNF);
	return grammar;
}

EpsilonFreeCFG GrammarFromXMLParser::parseEpsilonFreeCFG(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_EPSILON_FREE_CFG);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	EpsilonFreeCFG grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	bool generatesEpsilon = parseGeneratesEpsilon(input);
	grammar.setGeneratesEpsilon(generatesEpsilon);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_EPSILON_FREE_CFG);
	return grammar;
}

CFG GrammarFromXMLParser::parseCFG(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_CFG);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	CFG grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_CFG);
	return grammar;
}

LG GrammarFromXMLParser::parseLG(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_LG);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	LG grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_LG);
	return grammar;
}

RightRG GrammarFromXMLParser::parseRightRG(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_RIGHT_RG);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	RightRG grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	bool generatesEpsilon = parseGeneratesEpsilon(input);
	grammar.setGeneratesEpsilon(generatesEpsilon);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_RIGHT_RG);
	return grammar;
}

RightLG GrammarFromXMLParser::parseRightLG(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_RIGHT_LG);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	RightLG grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_RIGHT_LG);
	return grammar;
}

LeftRG GrammarFromXMLParser::parseLeftRG(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_LEFT_RG);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	LeftRG grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	bool generatesEpsilon = parseGeneratesEpsilon(input);
	grammar.setGeneratesEpsilon(generatesEpsilon);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_LEFT_RG);
	return grammar;
}

LeftLG GrammarFromXMLParser::parseLeftLG(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::GRAMMAR_LEFT_LG);

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	LeftLG grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules(input, grammar);

	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::GRAMMAR_LEFT_LG);
	return grammar;
}


std::set<alphabet::Symbol> GrammarFromXMLParser::parseNonterminalAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> inputSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "nonterminalAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		inputSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "nonterminalAlphabet");
	return inputSymbols;
}

std::set<alphabet::Symbol> GrammarFromXMLParser::parseTerminalAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> inputSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "terminalAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		inputSymbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "terminalAlphabet");
	return inputSymbols;
}

alphabet::Symbol GrammarFromXMLParser::parseInitialSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "initialSymbol");
	alphabet::Symbol blank(alib::xmlApi<alphabet::Symbol>::parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "initialSymbol");
	return blank;
}

bool GrammarFromXMLParser::parseGeneratesEpsilon(std::list<sax::Token>& input) const {
	bool generatesEpsilon;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "generatesEpsilon");
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "true")) {
		input.pop_front();
		popToken(input, sax::Token::TokenType::END_ELEMENT, "true");
		generatesEpsilon = true;
	} else {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "false");
		popToken(input, sax::Token::TokenType::END_ELEMENT, "false");
		generatesEpsilon = false;
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "generatesEpsilon");
	return generatesEpsilon;
}

template<class T>
void GrammarFromXMLParser::parseRules(std::list<sax::Token> &input, T& grammar) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "rules");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "rule");
		parseRule(input, grammar);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "rule");
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "rules");
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, UnrestrictedGrammar& grammar) const {
	std::vector<alphabet::Symbol> lhs = parseRuleLHS(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, ContextPreservingUnrestrictedGrammar& grammar) const {
	std::vector<alphabet::Symbol> lContext = parseRuleLContext(input);
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::vector<alphabet::Symbol> rContext = parseRuleRContext(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRule(lContext, lhs, rContext, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, NonContractingGrammar& grammar) const {
	std::vector<alphabet::Symbol> lhs = parseRuleLHS(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, CSG& grammar) const {
	std::vector<alphabet::Symbol> lContext = parseRuleLContext(input);
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::vector<alphabet::Symbol> rContext = parseRuleRContext(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRule(lContext, lhs, rContext, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, GNF& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>> rhs = parseRuleGNFRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, CNF& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>> rhs = parseRuleOneOrTwoSymbolsRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, EpsilonFreeCFG& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, CFG& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, LG& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRawRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, RightRG& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>> rhs = parseRuleOneOrTwoSymbolsRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, RightLG& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRawRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, LeftRG& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>> rhs = parseRuleOneOrTwoSymbolsRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, LeftLG& grammar) const {
	alphabet::Symbol lhs = parseRuleSingleSymbolLHS(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRawRule(lhs, rhs);
}


std::vector<alphabet::Symbol> GrammarFromXMLParser::parseRuleLContext(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> lContext;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "lContext");
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "epsilon")) {
		input.pop_front();
		popToken(input, sax::Token::TokenType::END_ELEMENT, "epsilon");
	} else while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		lContext.push_back(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "lContext");
	return lContext;
}

std::vector<alphabet::Symbol> GrammarFromXMLParser::parseRuleLHS(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> lhs;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "lhs");
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "epsilon")) {
		input.pop_front();
		popToken(input, sax::Token::TokenType::END_ELEMENT, "epsilon");
	} else while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		lhs.push_back(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "lhs");
	return lhs;
}

alphabet::Symbol GrammarFromXMLParser::parseRuleSingleSymbolLHS(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "lhs");
	alphabet::Symbol lhs = alib::xmlApi<alphabet::Symbol>::parse(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "lhs");
	return lhs;
}

std::vector<alphabet::Symbol> GrammarFromXMLParser::parseRuleRContext(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> rContext;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "rContext");
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "epsilon")) {
		input.pop_front();
		popToken(input, sax::Token::TokenType::END_ELEMENT, "epsilon");
	} else while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		rContext.push_back(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "rContext");
	return rContext;
}

std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>> GrammarFromXMLParser::parseRuleGNFRHS(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "rhs");
	alphabet::Symbol first = alib::xmlApi<alphabet::Symbol>::parse(input);
	std::vector<alphabet::Symbol> second;
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		second.push_back(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "rhs");
	return std::make_pair(first, second);
}

std::vector<alphabet::Symbol> GrammarFromXMLParser::parseRuleRHS(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> rhs;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "rhs");
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "epsilon")) {
		input.pop_front();
		popToken(input, sax::Token::TokenType::END_ELEMENT, "epsilon");
	} else while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		rhs.push_back(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "rhs");
	return rhs;
}

std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>> GrammarFromXMLParser::parseRuleOneOrTwoSymbolsRHS(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "rhs");
	alphabet::Symbol first = alib::xmlApi<alphabet::Symbol>::parse(input);
	if(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		alphabet::Symbol second = alib::xmlApi<alphabet::Symbol>::parse(input);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "rhs");
		return std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>>(std::make_pair(first, second));
	} else {
		popToken(input, sax::Token::TokenType::END_ELEMENT, "rhs");
		return std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>>(first);
	}
}

} /* namespace grammar */

