/*
 * PrimitiveFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveFromXMLParser.h"
#include "../sax/ParserException.h"
#include <algorithm>
#include "../std/stou.h"

#include "../XmlApi.hpp"

namespace primitive {

Primitive PrimitiveFromXMLParser::parsePrimitive(std::list<sax::Token>& input) const {
	return parsePrimitive(input, std::set<FEATURES>({FEATURES::STRING, FEATURES::CHAR, FEATURES::INTEGER}));
}

Primitive PrimitiveFromXMLParser::parsePrimitive(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(alib::xmlApi<Integer>::first(input)) {
		if(!features.count(FEATURES::INTEGER)) throw exception::AlibException();
		return Primitive(parseInteger(input));
	} else if(alib::xmlApi<String>::first(input)) {
		if(!features.count(FEATURES::STRING)) throw exception::AlibException();
		return Primitive(parseString(input));
	} else if(alib::xmlApi<Character>::first(input)) {
		if(!features.count(FEATURES::CHAR)) throw exception::AlibException();
		return Primitive(parseCharacter(input));
	} else if(alib::xmlApi<Unsigned>::first(input)) {
		if(!features.count(FEATURES::UNSIGNED)) throw exception::AlibException();
		return Primitive(parseUnsigned(input));
	} else if(alib::xmlApi<Bool>::first(input)) {
		if(!features.count(FEATURES::BOOL)) throw exception::AlibException();
		return Primitive(parseBool(input));
	} else {
		throw sax::ParserException(sax::Token("Integer, String, Character, Unsigned, Bool", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

bool PrimitiveFromXMLParser::first(const std::list<sax::Token>& input) const {
	if(alib::xmlApi<Integer>::first(input) || alib::xmlApi<String>::first(input) || alib::xmlApi<Character>::first(input) || alib::xmlApi<Unsigned>::first(input) || alib::xmlApi<Bool>::first(input)) {
		return true;
	} else {
		return false;
	}
}

Integer PrimitiveFromXMLParser::parseInteger(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_INTEGER);
	Integer data(std::stoi(popTokenData(input, sax::Token::TokenType::CHARACTER)));
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_INTEGER);
	return data;
}

Character PrimitiveFromXMLParser::parseCharacter(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_CHARACTER);
	Character data(popTokenData(input, sax::Token::TokenType::CHARACTER)[0]);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_CHARACTER);
	return data;
}

String PrimitiveFromXMLParser::parseString(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_STRING);
	String data(popTokenData(input, sax::Token::TokenType::CHARACTER));
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_STRING);
	return data;
}

Unsigned PrimitiveFromXMLParser::parseUnsigned(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_UNSIGNED);
	Unsigned data(std::stou(popTokenData(input, sax::Token::TokenType::CHARACTER)));
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_UNSIGNED);
	return data;
}

Bool PrimitiveFromXMLParser::parseBool(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_BOOL);
	std::string tmp = popTokenData(input, sax::Token::TokenType::CHARACTER);
	bool data;
	if(tmp == "true")
		data = true;
	else if(tmp == "false")
		data = false;
	else
		throw exception::AlibException("Invalid boolean value");
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_BOOL);
	return Bool(data);
}

int PrimitiveFromXMLParser::parseIntegerRaw(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_INTEGER);
	int data = std::stoi(popTokenData(input, sax::Token::TokenType::CHARACTER));
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_INTEGER);
	return data;
}

char PrimitiveFromXMLParser::parseCharacterRaw(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_CHARACTER);
	char data = popTokenData(input, sax::Token::TokenType::CHARACTER)[0];
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_CHARACTER);
	return data;
}

std::string PrimitiveFromXMLParser::parseStringRaw(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_STRING);
	std::string data = popTokenData(input, sax::Token::TokenType::CHARACTER);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_STRING);
	return data;
}

unsigned PrimitiveFromXMLParser::parseUnsignedRaw(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_UNSIGNED);
	unsigned data = std::stou(popTokenData(input, sax::Token::TokenType::CHARACTER));
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_UNSIGNED);
	return data;
}

bool PrimitiveFromXMLParser::parseBoolRaw(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::PRIMITIVE_BOOL);
	std::string tmp = popTokenData(input, sax::Token::TokenType::CHARACTER);
	bool data;
	if(tmp == "true")
		data = true;
	else if(tmp == "false")
		data = false;
	else
		throw exception::AlibException("Invalid boolean value");
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::PRIMITIVE_BOOL);
	return data;
}

} /* namespace primitive */
