/*
 * RegExpFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "RegExpFromStringParser.h"
#include "../sax/ParserException.h"
#include "../exception/AlibException.h"

#include "../StringApi.hpp"

namespace regexp {

RegExp RegExpFromStringParser::parseRegExp(std::istream& input) const {
	return parseRegExp(input, std::set<FEATURES>({FEATURES::FORMAL, FEATURES::UNBOUNDED}));
}

RegExp RegExpFromStringParser::parseRegExp(std::istream& input, const std::set<FEATURES>& features) const {
	UnboundedRegExpElement* element = this->alternation(input);
	UnboundedRegExp regexp(std::move(*element));
	delete element;
	if(features.count(FEATURES::UNBOUNDED)) return RegExp{regexp};

	if(features.count(FEATURES::FORMAL)) return RegExp{FormalRegExp{regexp}};

	throw exception::AlibException("Invalid input");
}

UnboundedRegExpElement* RegExpFromStringParser::alternation(std::istream& input) const {
	return this->alternationCont(input, this->concatenation(input));
}

UnboundedRegExpElement* RegExpFromStringParser::alternationCont(std::istream& input, UnboundedRegExpElement* left) const {
	RegExpFromStringLexer::Token token = m_RegexpLexer.next(input);
	if(token.type == RegExpFromStringLexer::TokenType::PLUS) {
		try {
			UnboundedRegExpElement* right = this->concatenation(input);
			UnboundedRegExpAlternation* res = new UnboundedRegExpAlternation();
			res->appendElement(std::move(*left));
			res->appendElement(std::move(*right));
			delete right;
			delete left;

			return this->alternationContCont(input, res);
		} catch (...) {
			delete left;
			throw;
		}
	} else {
		m_RegexpLexer.putback(input, token);
		return left;
	}
}

UnboundedRegExpElement* RegExpFromStringParser::alternationContCont(std::istream& input, UnboundedRegExpAlternation* res) const {
	RegExpFromStringLexer::Token token = m_RegexpLexer.next(input);
	if(token.type == RegExpFromStringLexer::TokenType::PLUS) {
		try {
			UnboundedRegExpElement* next = this->concatenation(input);
			res->appendElement(std::move(*next));
			delete next;

			return this->alternationContCont(input, res);
		} catch (...) {
			delete res;
			throw;
		}
	} else {
		m_RegexpLexer.putback(input, token);
		return res;
	}
}


UnboundedRegExpElement* RegExpFromStringParser::concatenation(std::istream& input) const {
	return this->concatenationCont(input, this->factor(input));
}

UnboundedRegExpElement* RegExpFromStringParser::concatenationCont(std::istream& input, UnboundedRegExpElement* left) const {
	RegExpFromStringLexer::Token token = m_RegexpLexer.next(input);
	if(token.type == RegExpFromStringLexer::TokenType::ERROR || token.type == RegExpFromStringLexer::TokenType::LPAR || token.type == RegExpFromStringLexer::TokenType::EPS || token.type == RegExpFromStringLexer::TokenType::EMPTY) {
		try {
			m_RegexpLexer.putback(input, token);
			UnboundedRegExpElement* right = this->factor(input);
			UnboundedRegExpConcatenation* res = new UnboundedRegExpConcatenation();
			res->appendElement(std::move(*left));
			res->appendElement(std::move(*right));
			delete right;
			delete left;

			return this->concatenationContCont(input, res);
		} catch (...) {
			delete left;
			throw;
		}
	} else {
		m_RegexpLexer.putback(input, token);
		return left;
	}
	return NULL; //unreachable but produces warning if not present
}

UnboundedRegExpElement* RegExpFromStringParser::concatenationContCont(std::istream& input, UnboundedRegExpConcatenation* res) const {
	RegExpFromStringLexer::Token token = m_RegexpLexer.next(input);
	if(token.type == RegExpFromStringLexer::TokenType::ERROR || token.type == RegExpFromStringLexer::TokenType::LPAR || token.type == RegExpFromStringLexer::TokenType::EPS || token.type == RegExpFromStringLexer::TokenType::EMPTY) {
		try {
			m_RegexpLexer.putback(input, token);
			UnboundedRegExpElement* next = this->factor(input);
			res->appendElement(std::move(*next));
			delete next;

			return this->concatenationContCont(input, res);
		} catch (...) {
			delete res;
			throw;
		}
	} else {
		m_RegexpLexer.putback(input, token);
		return res;
	}
	return NULL; //unreachable but produces warning if not present
}

UnboundedRegExpElement* RegExpFromStringParser::factor(std::istream& input) const {
	RegExpFromStringLexer::Token token = m_RegexpLexer.next(input);
	if(token.type == RegExpFromStringLexer::TokenType::LPAR) {
		UnboundedRegExpElement* base = this->alternation(input);
		token = m_RegexpLexer.next(input);
		if(token.type != RegExpFromStringLexer::TokenType::RPAR) throw exception::AlibException("Expected RPAR");
		return this->star(input, base);
	} else if(token.type == RegExpFromStringLexer::TokenType::EPS) {
		return this->star(input, new UnboundedRegExpEpsilon());
	} else if(token.type == RegExpFromStringLexer::TokenType::EMPTY) {
		return this->star(input, new UnboundedRegExpEmpty());
	} else if(token.type == RegExpFromStringLexer::TokenType::ERROR) {
		UnboundedRegExpSymbol* res = new UnboundedRegExpSymbol(alib::stringApi<alphabet::Symbol>::parse(input));
		return this->star(input, res);
	} else {
		throw exception::AlibException("Unrecognised token at factor rule");
	}
}

UnboundedRegExpElement* RegExpFromStringParser::star(std::istream& input, UnboundedRegExpElement* elem) const {
	RegExpFromStringLexer::Token token = m_RegexpLexer.next(input);
	if(token.type == RegExpFromStringLexer::TokenType::STAR) {
		UnboundedRegExpIteration* iter = new UnboundedRegExpIteration(std::move(*elem));
		delete elem;
		return this->star(input, iter);
	} else {
		m_RegexpLexer.putback(input, token);
		return elem;
	}
}

}
