/*
 * FormalRegExpIteration.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "FormalRegExpIteration.h"
#include "../../exception/AlibException.h"

#include "../unbounded/UnboundedRegExpIteration.h"
#include <sstream>

namespace regexp {

FormalRegExpIteration::FormalRegExpIteration(FormalRegExpElement&& element) : element( NULL ) {
	this->setElement(std::move(element));
}

FormalRegExpIteration::FormalRegExpIteration(const FormalRegExpElement& element) : element( NULL ) {
	this->setElement(element);
}


FormalRegExpIteration::FormalRegExpIteration(const FormalRegExpIteration& other) : element(other.element->clone()) {

}

FormalRegExpIteration::FormalRegExpIteration(FormalRegExpIteration&& other) noexcept : element(other.element) {
	other.element = NULL;
	this->attachRegExp(NULL);
}

FormalRegExpIteration& FormalRegExpIteration::operator=(const FormalRegExpIteration& other) {
	if (this == &other) {
		return *this;
	}

	*this = FormalRegExpIteration(other);

	return *this;
}

FormalRegExpIteration& FormalRegExpIteration::operator=(FormalRegExpIteration&& other) {
	std::swap(this->element, other.element);
	std::swap(this->parentRegExp, other.parentRegExp);
	
	this->attachRegExp(other.parentRegExp);

	return *this;
}

regexp::FormalRegExpIteration::~FormalRegExpIteration() noexcept {
	delete element;
}

const FormalRegExpElement & FormalRegExpIteration::getElement() const {
	return *element;
}

FormalRegExpElement & FormalRegExpIteration::getElement() {
	return *element;
}

void FormalRegExpIteration::setElement(const FormalRegExpElement& element) {
	FormalRegExpElement* elem = element.clone();
	if(this->parentRegExp && !this->element->attachRegExp(this->parentRegExp))
		throw exception::AlibException("Input symbols not in the alphabet.");
	delete this->element;
	this->element = elem;
}

void FormalRegExpIteration::setElement(FormalRegExpElement&& element) {
	FormalRegExpElement* elem = std::move(element).plunder();
	if(this->parentRegExp && !this->element->attachRegExp(this->parentRegExp))
		throw exception::AlibException("Input symbols not in the alphabet."); 
	delete this->element;
	this->element = elem;
}

FormalRegExpElement* FormalRegExpIteration::clone() const {
	return new FormalRegExpIteration(*this);
}

FormalRegExpElement* FormalRegExpIteration::plunder() && {
	return new FormalRegExpIteration(std::move(*this));
}

UnboundedRegExpElement* FormalRegExpIteration::cloneAsUnbounded() const {
	UnboundedRegExpElement* tmp = this->element->cloneAsUnbounded();
	UnboundedRegExpElement* res = new UnboundedRegExpIteration(std::move(*tmp));
	delete tmp;
	return res;
}

int FormalRegExpIteration::compare(const FormalRegExpIteration& other) const {
	return element->compare(*other.element);
}

void FormalRegExpIteration::operator>>(std::ostream& out) const {
	out << "(RegExpFormalRegExpIteration " << *element << ")";
}

bool FormalRegExpIteration::testSymbol( const alphabet::Symbol & symbol ) const {
	return element->testSymbol( symbol );
}

bool FormalRegExpIteration::attachRegExp(const FormalRegExp * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	return this->element->attachRegExp(regexp);
}

void FormalRegExpIteration::computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const {
	element->computeMinimalAlphabet(alphabet);
}

FormalRegExpIteration::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace regexp */

