/*
 * UnboundedRegExpConcatenation.cpp
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpConcatenation.h"
#include "../../exception/AlibException.h"
#include "../formal/FormalRegExpConcatenation.h"
#include "../formal/FormalRegExpEpsilon.h"
#include <sstream>

namespace regexp {

UnboundedRegExpConcatenation::UnboundedRegExpConcatenation() {
}

UnboundedRegExpConcatenation::UnboundedRegExpConcatenation(const UnboundedRegExpConcatenation& other) {
	for (auto element : other.elements) {
		elements.push_back(element->clone());
	}
}

UnboundedRegExpConcatenation::UnboundedRegExpConcatenation(UnboundedRegExpConcatenation&& other) noexcept : elements(std::move(other.elements)) {
	other.elements.clear();
	this->attachRegExp(NULL);
}

UnboundedRegExpConcatenation& UnboundedRegExpConcatenation::operator=(const UnboundedRegExpConcatenation& other) {
	if(this == &other) {
		return *this;
	}

	*this = UnboundedRegExpConcatenation(other);
	
	return *this;
}

UnboundedRegExpConcatenation& UnboundedRegExpConcatenation::operator=(UnboundedRegExpConcatenation&& other) {
	std::swap(this->elements, other.elements);
	std::swap(this->parentRegExp, other.parentRegExp); // this->parentRegExp is stored within othes.parentRegExp and it is reattached on the next line

	this->attachRegExp(other.parentRegExp);
	
	return *this;
}

UnboundedRegExpConcatenation::~UnboundedRegExpConcatenation() noexcept {
	for (auto element : elements) {
		delete element;
	}
	elements.clear();
}

const std::vector<const UnboundedRegExpElement*> & UnboundedRegExpConcatenation::getElements() const {
	return * reinterpret_cast<const std::vector<const UnboundedRegExpElement*> * > (&elements);
}

const std::vector<UnboundedRegExpElement*> & UnboundedRegExpConcatenation::getElements() {
	return elements;
}

void UnboundedRegExpConcatenation::appendElement(const UnboundedRegExpElement& element) {
	UnboundedRegExpElement* elem = element.clone();
	if(this->parentRegExp && !elem->attachRegExp(this->parentRegExp))
		throw exception::AlibException("Input symbols not in the alphabet."); 
	this->elements.push_back(elem);
}

void UnboundedRegExpConcatenation::appendElement(UnboundedRegExpElement&& element) {
	UnboundedRegExpElement* elem = std::move(element).plunder();
	if(this->parentRegExp && !elem->attachRegExp(this->parentRegExp))
		throw exception::AlibException("Input symbols not in the alphabet."); 
	this->elements.push_back(elem);
}

UnboundedRegExpElement* UnboundedRegExpConcatenation::clone() const {
	return new UnboundedRegExpConcatenation(*this);
}

UnboundedRegExpElement* UnboundedRegExpConcatenation::plunder() && {
	return new UnboundedRegExpConcatenation(std::move(*this));
}

FormalRegExpElement* UnboundedRegExpConcatenation::cloneAsFormal() const {
	if(elements.size() == 0) return new FormalRegExpEpsilon();
	if(elements.size() == 1) return elements[0]->cloneAsFormal();

	int size = elements.size();
	FormalRegExpElement* rightTmp = elements[--size]->cloneAsFormal();
	FormalRegExpElement* leftTmp = elements[--size]->cloneAsFormal();

	FormalRegExpConcatenation* res = new FormalRegExpConcatenation(std::move(*leftTmp), std::move(*rightTmp));
	delete rightTmp;
	delete leftTmp;

	while(size) {
		FormalRegExpElement* tmp = res;
		FormalRegExpElement* newLeft = elements[--size]->cloneAsFormal();

		res = new FormalRegExpConcatenation(std::move(*newLeft), std::move(*tmp));
		delete tmp;
		delete newLeft;
	}

	return res;
}

int UnboundedRegExpConcatenation::compare(const UnboundedRegExpConcatenation& other) const {
	int thisSize = this->elements.size();
	int otherSize = other.elements.size();
	if(thisSize < otherSize) return -1;
	if(thisSize > otherSize) return 1;

	auto thisIter = this->elements.begin();
	auto otherIter = other.elements.begin();
	for(; thisIter != this->elements.end(); thisIter++, otherIter++) {
		int res = (*thisIter)->compare(**otherIter);
		if(res != 0) return res;
	}
	return 0;
}

void UnboundedRegExpConcatenation::operator>>(std::ostream& out) const {
	out << "(UnboundedRegExpConcatenation";
	for(const auto& e : elements)
		out << " " << *e;

	out << ")";
}

bool UnboundedRegExpConcatenation::testSymbol( const alphabet::Symbol & symbol ) const {
	for(const auto& child : this->elements)
		if(child->testSymbol(symbol)) return true;
	return false;
}

bool UnboundedRegExpConcatenation::attachRegExp(const UnboundedRegExp * regexp ) {
	if(this->parentRegExp == regexp) return true;
	
	this->parentRegExp = regexp;
	for(const auto& child : this->elements)
		if(!child->attachRegExp(regexp)) return false;
	return true;
}

void UnboundedRegExpConcatenation::computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const {
	for(const auto& child : this->elements)
		child->computeMinimalAlphabet(alphabet);
}

UnboundedRegExpConcatenation::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace regexp */
