/*
 * UnboundedRegExpIteration.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpIteration.h"
#include "../../exception/AlibException.h"

#include "../formal/FormalRegExpIteration.h"
#include <sstream>

namespace regexp {

UnboundedRegExpIteration::UnboundedRegExpIteration(UnboundedRegExpElement&& element) : element( NULL ) {
	this->setElement(std::move(element));
}

UnboundedRegExpIteration::UnboundedRegExpIteration(const UnboundedRegExpElement& element) : element( NULL ) {
	this->setElement(element);
}


UnboundedRegExpIteration::UnboundedRegExpIteration(const UnboundedRegExpIteration& other) : element(other.element->clone()) {

}

UnboundedRegExpIteration::UnboundedRegExpIteration(UnboundedRegExpIteration&& other) noexcept : element(other.element) {
	other.element = NULL;
	this->attachRegExp(NULL);
}

UnboundedRegExpIteration& UnboundedRegExpIteration::operator=(const UnboundedRegExpIteration& other) {
	if (this == &other) {
		return *this;
	}

	*this = UnboundedRegExpIteration(other);

	return *this;
}

UnboundedRegExpIteration& UnboundedRegExpIteration::operator=(UnboundedRegExpIteration&& other) {
	std::swap(this->element, other.element);
	std::swap(this->parentRegExp, other.parentRegExp);
	
	this->attachRegExp(other.parentRegExp);

	return *this;
}

regexp::UnboundedRegExpIteration::~UnboundedRegExpIteration() noexcept {
	delete element;
}

const UnboundedRegExpElement & UnboundedRegExpIteration::getElement() const {
	return *element;
}

UnboundedRegExpElement & UnboundedRegExpIteration::getElement() {
	return *element;
}

void UnboundedRegExpIteration::setElement(const UnboundedRegExpElement& element) {
	UnboundedRegExpElement* elem = element.clone();
	if(this->parentRegExp && !this->element->attachRegExp(this->parentRegExp))
		throw exception::AlibException("Input symbols not in the alphabet.");
	delete this->element;
	this->element = elem;
}

void UnboundedRegExpIteration::setElement(UnboundedRegExpElement&& element) {
	UnboundedRegExpElement* elem = std::move(element).plunder();
	if(this->parentRegExp && !this->element->attachRegExp(this->parentRegExp))
		throw exception::AlibException("Input symbols not in the alphabet."); 
	delete this->element;
	this->element = elem;
}

UnboundedRegExpElement* UnboundedRegExpIteration::clone() const {
	return new UnboundedRegExpIteration(*this);
}

UnboundedRegExpElement* UnboundedRegExpIteration::plunder() && {
	return new UnboundedRegExpIteration(std::move(*this));
}

FormalRegExpElement* UnboundedRegExpIteration::cloneAsFormal() const {
	FormalRegExpElement* tmp = this->element->cloneAsFormal();
	FormalRegExpElement* res = new FormalRegExpIteration(std::move(*tmp));
	delete tmp;
	return res;
}

int UnboundedRegExpIteration::compare(const UnboundedRegExpIteration& other) const {
	return element->compare(*other.element);
}

void UnboundedRegExpIteration::operator>>(std::ostream& out) const {
	out << "(RegExpUnboundedRegExpIteration " << *element << ")";
}

bool UnboundedRegExpIteration::testSymbol( const alphabet::Symbol & symbol ) const {
	return element->testSymbol( symbol );
}

bool UnboundedRegExpIteration::attachRegExp(const UnboundedRegExp * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	return this->element->attachRegExp(regexp);
}

void UnboundedRegExpIteration::computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const {
	element->computeMinimalAlphabet(alphabet);
}

UnboundedRegExpIteration::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace regexp */

