/*
 * LinearString.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LinearString.h"
#include "../exception/AlibException.h"

#include <cassert>
#include <sstream>
#include <algorithm>
#include <typeinfo>
#include <tuple>

#include "CyclicString.h"
#include "Epsilon.h"

namespace string {

LinearString::LinearString() {

}

LinearString::LinearString(const std::set<alphabet::Symbol>& alphabet, const std::vector<alphabet::Symbol>& data) {
	this->alphabet = alphabet;
	setContent(data);
}

LinearString::LinearString(std::set<alphabet::Symbol>&& alphabet, std::vector<alphabet::Symbol>&& data) {
	this->alphabet = std::move(alphabet);
	setContent(std::move(data));
}

LinearString::LinearString(const std::vector<alphabet::Symbol>& data) {
	alphabet = std::set<alphabet::Symbol>(data.begin(), data.end());
	m_Data = data;
}

LinearString::LinearString(std::vector<alphabet::Symbol>&& data) {
	alphabet = std::set<alphabet::Symbol>(data.begin(), data.end());
	m_Data = std::move(data);
}

LinearString::LinearString(const std::string& str) {
	for (unsigned i = 0; i < str.length(); ++i) {
		m_Data.push_back(alphabet::symbolFrom(str[i]));
	}
	alphabet = std::set<alphabet::Symbol>(m_Data.begin(), m_Data.end());
}

LinearString::LinearString(const Epsilon& epsilon) {
	alphabet = epsilon.getAlphabet();
}

StringBase* LinearString::clone() const {
	return new LinearString(*this);
}

StringBase* LinearString::plunder() && {
	return new LinearString(std::move(*this));
}

bool LinearString::removeSymbolFromAlphabet(const alphabet::Symbol & symbol) {
	if(std::any_of(m_Data.begin(), m_Data.end(), [&](const alphabet::Symbol & s) { return s == symbol; } ) )
		throw exception::AlibException("Input symbol \"" + (std::string) symbol + "\" is used.");
	
	return alphabet.erase(symbol);
}

void LinearString::appendSymbol(const alphabet::Symbol& symbol) {
	if(alphabet.count(symbol) == 0)
		throw exception::AlibException("Input symbol \"" + (std::string) symbol + "\" not in the alphabet.");
	m_Data.push_back(symbol);
}

void LinearString::appendSymbol(alphabet::Symbol&& symbol) {
	if(alphabet.count(symbol) == 0)
		throw exception::AlibException("Input symbol \"" + (std::string) symbol + "\" not in the alphabet.");
	m_Data.push_back(std::move(symbol));
}

const std::vector<alphabet::Symbol>& LinearString::getContent() const {
	return this->m_Data;
}

void LinearString::setContent(std::vector<alphabet::Symbol> data) {
	std::set<alphabet::Symbol> minimalAlphabet(data.begin(), data.end());
	std::set<alphabet::Symbol> unknownSymbols;
	std::set_difference(minimalAlphabet.begin(), minimalAlphabet.end(), alphabet.begin(), alphabet.end(), std::inserter(unknownSymbols, unknownSymbols.end()));

	if(unknownSymbols.size() > 0)
		throw exception::AlibException("Input symbols not in the alphabet.");

	this->m_Data = std::move(data);
}

bool LinearString::isEmpty() const {
	return this->m_Data.size() == 0;
}

int LinearString::compare(const LinearString& other) const {
	auto first = std::tie(m_Data, alphabet);
	auto second = std::tie(other.m_Data, other.alphabet);

	std::compare<decltype(first)> comp;
	return comp(first, second);
}

int LinearString::compare(const CyclicString& other) const {
	if(this->isEmpty() && other.isEmpty()) {
		std::compare<std::set<alphabet::Symbol>> comp;
		return comp(alphabet, other.getAlphabet());
	}

	if(this->selfTypeId() < typeId<CyclicString>())
		return -1;
	else
		return 1;
}

int LinearString::compare(const Epsilon& other) const {
	if(this->isEmpty()) {
		std::compare<std::set<alphabet::Symbol>> comp;
		return comp(alphabet, other.getAlphabet());
	}
	
	if(this->selfTypeId() < typeId<Epsilon>())
		return -1;
	else
		return 1;
}

void LinearString::operator >>(std::ostream& out) const {
	if( this->isEmpty() ) {
		out << "(Epsilon)";
	} else {
		out << "(LinearString ";
		for(const alphabet::Symbol& symbol : this->m_Data)
			out << symbol;
		out << ")";
	}
}

LinearString::operator std::string () const {
	std::stringstream ss;
	if( this->isEmpty() ) {
		ss << "E";
	} else {
		ss << "\"";
		for(const alphabet::Symbol& symbol : this->m_Data)
			ss << (std::string) symbol;
		ss << "\"";
	}
	return std::move(ss).str();
}

} /* namespace string */

